unit DlgRemote;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Mask, Spin, ComCtrls, FileCtrl,
  Buttons, Inifiles, AppEvnts, Registry,
  ThreadMessage, OleCtrls, CamRemoteActiveXXControl1_TLB, CamDefines;

const REMOTE_SUBKEY         = 'RemoteTemplate'; //Key used when reading writing to registry
      THUMBNAIL_FILENAME    = '\thumbnail.jpg'; //Thumbnail name
      LOG_FILE_NAME         = '\log.txt';       //Log-file name
      MAX_LOG_SIZE          = 65000;            //Max size of log-file
      START_REMOTE_TXT      = '&Start taking pictures remote';
      STOP_REMOTE_TXT       = '&Stop taking pictures remote';
      START_REMOTE_PROC_TXT = '&Start remote process';
      STOP_REMOTE_PROC_TXT  = '&Stop remote process';
      REMOTE_MODE           = ReleaseDataKingTakeBothThumbAndPic; //or ReleaseDataKindTakeOnlyPicture
      LOCK_AF_TXT           = '&Lock AF';
      UNLOCK_AF_TXT         = 'Un&lock AF';
      KELVIN_TEXT           = 'Kelvin';
      RAW_TEXT              = 'RAW';
      REMOTE_PARAM_FILE     = 'RemoteParamFile.dta';

//List all supported cameras, which is sused to separatelu define supported
//parameters per camera. This function is however done by CamRemote when
//starting the remote process.
type AvailRemoteCamerasType =
       (
         AllParameters,
         ConnectedCameraParameters
       );

//String corresponding type to AvailRemoteCamerasType
const RemoteCamerasStringType : array[AvailRemoteCamerasType] of string =
       ( 'All parameters',
         'Connected camera parameters'
       );

//A record of supported remote parameters
type CameraRemoteParamType =
           record
             CompQuality         : array of RemoteFormatQualityType;
             CompQualityPic2RAW  : array of RemoteFormatQualityType;
             ImageSize           : array of RemoteFormatSizeType;
             ImageSizePic2RAW    : array of RemoteFormatSizeType;
             StrobeSetting       : array of RemoteFormatFlashType;
             StrobeCompSetting   : array of RemoteFormatFlashCompType;
             ImageMode           : array of RemoteFormatShootingModeType;
             DriveMode           : array of RemoteFormatDriveModeType;
             MLWeiMode           : array of RemoteFormatMLWeiType;
             AFMode              : array of RemoteFormatAFModeType;
             AFDistance          : array of RemoteFormatAFDistType;
             AFFocusingPoint     : array of RemoteFormatAFFocusingPointType;
             AFAssistLight       : array of RemoteFormatAFLightType;
             WhiteBalanceSetting : array of RemoteFormatWBType;
             Contrast            : array of RemoteFormatLevelType;
             ColorGain           : array of RemoteFormatLevelType;
             Sharpness           : array of RemoteFormatLevelType;
             ColorSpace          : array of RemoteFormatColorSpaceType;
             ISO                 : array of RemoteFormatISOType;
             ExposureComp        : array of RemoteFormatExposureCompType;
             AV                  : array of RemoteFormatAVType;
             TV                  : array of RemoteFormatTVType;
             PhotoEffect         : array of RemoteFormatPhotoEffectType;
             Beep                : array of RemoteFormatBeepType;
           end;

//------------------------------------------------------------------------------
//- Global variables including texts used in the comboboxes when handling
//- remote paramters
//------------------------------------------------------------------------------
var gCompQualArrayText : array[low(RemoteFormatQualityType)..high(RemoteFormatQualityType)] of string =
  ('-', 'Economy', 'Normal', 'Fine', 'Super Fine', RAW_TEXT);

var gImageSizeArrayText : array[low(RemoteFormatSizeType)..high(RemoteFormatSizeType)] of string =
  ('-', 'Large', 'Medium',  'Small', 'Medium1', 'Medium2', 'Medium3');

var gStrobeSettingArrayText : array[low(RemoteFormatFlashType)..high(RemoteFormatFlashType)] of string =
  ('-', 'Off', 'Auto', 'On', 'auto+red eye', 'auto+slow sync', 'auto+red eye', 'on+red eye');

var gStrobeCompSettingArrayText : array[low(RemoteFormatFlashCompType)..high(RemoteFormatFlashCompType)] of string =
  ('-', '+2.00', '+1.66', '+1.33', '+1.00', '+0.66', '+0.33', '0',
   '-0.33', '-0.66', '-1.00', '-1.33', '-1.66', '-2.00');

var gImageModeArrayText : array[low(RemoteFormatShootingModeType)..high(RemoteFormatShootingModeType)] of string =
 ('-',            'Auto',          'Manual mode',     'Far Scene',      'Fast Shutter',
  'Slow Shutter', 'Night Scene',   'Gray Scene',      'Sepia',          'Portrait',
  'Sport',        'Macro',         'Black and White', 'Pan Focus',      'Vivid',
  'Neutral',      'Program',       'TV',              'AV',             'ADep',
  'MDep',         'Bulb',          'Manual2',         'Flash Off',      'Long Shutter',
  'Super Macro',  'Foliage',       'Indoor',          'Fireworks',      'Beach',
  'Underwater',   'Snow',          'Kids and pets',   'Night snapshot', 'Ditigal macro',
  'My colors',    'Photo in movie'
  );

var gDriveModeArrayText : array[low(RemoteFormatDriveModeType)..high(RemoteFormatDriveModeType)] of string =
  ('-', 'Single frame', 'Continous', 'Video', 'High speed continous', 'Low speed continous',
   '10 sec self timer', '2 sec self timer');

var gMLWeiModeArrayText : array[low(RemoteFormatMLWeiType)..high(RemoteFormatMLWeiType)] of string =
  ('-', 'Center-weighted', 'Spot', 'Averaging', 'Evaluative', 'Partial',
   'Center-weighted averaging');

var gAFModeArrayText : array[low(RemoteFormatAFModeType)..high(RemoteFormatAFModeType)] of string =
  ('-', 'One-shot AF', 'AI Servo AF', 'AI Focus AF', 'Manual focus');

var gAFDistanceArrayText : array[low(RemoteFormatAFDistType)..high(RemoteFormatAFDistType)] of string =
  ('-', 'Manual', 'Auto', 'Unknown', 'Zone Focus(Close up)',
   'Zone Focus(The shortest distance)', 'Zone Focus(Short distance)',
   'Zone Focus(Medium distance)',       'Zone Focus(Far distance)',
   'Pan Focus', 'Super Macro', 'Infinity', 'Super Macro OCM');

var gAFFocusingPointArrayText : array[low(RemoteFormatAFFocusingPointType)..high(RemoteFormatAFFocusingPointType )] of string =
  ('-',
   'Center focus point, manual',    'Center focus point, auto',
   'Multiple focus points, manual', 'Multiple focus points, auto',
   'Multiple focus points, right',  'Multiple focus points, center',
   'Multiple focus points, left');

var gAFAssistLightArrayText : array[low(RemoteFormatAFLightType)..high(RemoteFormatAFLightType)] of string =
  ('-', 'Off', 'On');

var gWhiteBalanceSetArrayText : array[low(RemoteFormatWBType)..high(RemoteFormatWBType)] of string =
  ('-', 'Auto', 'Daylight', 'Cloudy', 'Tungsten', 'Fluorscent', 'Flash',
   'Fluorescent light', 'Custom', 'Custom1', 'Custom2', 'BW', 'Shade',
   KELVIN_TEXT, 'PCSet1', 'PCSet2', 'PCSet3');

var gContrastArrayText : array[low(RemoteFormatLevelType)..high(RemoteFormatLevelType)] of string =
  ('-', 'Low', 'Default', 'High');

var gColorGainArrayText : array[low(RemoteFormatLevelType)..high(RemoteFormatLevelType)] of string =
  ('-', 'Low', 'Default', 'High');

var gSharpnessArrayText : array[low(RemoteFormatLevelType)..high(RemoteFormatLevelType)] of string =
  ('-', 'Low', 'Default', 'High');

var gColorSpaceArrayText : array[low(RemoteFormatColorSpaceType)..high(RemoteFormatColorSpaceType)] of string =
  ('-', 'sRGB', 'AdobeRGB');

var gIsoArrayText : array[low(RemoteFormatISOType)..high(RemoteFormatISOType)] of string =
  ('-', 'Auto', '50',  '64',   '80',   '100',  '125',  '160',
                '200', '250',  '320',  '400',  '500',  '640' ,
                '800', '1000', '1250', '1600', '2000', '2500',
                '3200');

var gExposureCompArrayText : array[low(RemoteFormatExposureCompType)..high(RemoteFormatExposureCompType)] of string =
  ('-', '+2.00', '+1.66', '+1.33', '+1.00', '+0.66', '+0.33', '0',
   '-0.33', '-0.66', '-1.00', '-1.33', '-1.66', '-2.00');

var gAvArrayText : array[low(RemoteFormatAVType)..high(RemoteFormatAVType)] of string =
  ('-', '1.0', '1.1', '1.2', '1.4', '1.6', '1.8', '2.0', '2.2', '2.5',
   '2.8', '3.2', '3.5', '4.0', '4.5', '5.0', '5.6', '6.4', '7.1', '8.0',
   '9.0', '10', '11', '13', '14', '16', '18', '20', '22', '25', '29', '32',
   '36', '40', '45', '51', '57', '64', '72', '81', '91');

var gTvArrayText : array[low(RemoteFormatTVType)..high(RemoteFormatTVType)] of string =
  ('-', 'Bulb', '30"', '25"', '20"', '15"', '13"', '10"', '8"', '6"', '5"', '4"', '3"2', '2"5',
   '2"', '1"6', '1"3', '1"', '0"8', '0"6', '0"5', '0"4', '0"3', '1/4', '1/5', '1/6',
   '1/8', '1/10', '1/13', '1/15', '1/20', '1/25', '1/30', '1/40', '1/50',
   '1/60', '1/80', '1/100', '1/125', '1/160', '1/200', '1/250', '1/320', '1/400',
   '1/500', '1/640', '1/800', '1/1000', '1/1250', '1/1600', '1/2000', '1/2500', '1/3200',
   '1/4000', '1/5000', '1/6400', '1/8000', '1/10000', '1/12800', '1/16000');

var gPhotoEffectArrayText : array[low(RemoteFormatPhotoEffectType)..high(RemoteFormatPhotoEffectType)] of string =
  ('-', 'Off', 'Vivid', 'Neutral', 'Low Sharpening', 'Sepia', 'Black and White');

var gBeepArrayText : array[low(RemoteFormatBeepType)..high(RemoteFormatBeepType)] of string =
  ('-', 'Off', 'On');

//Remote form type
type
  TFormRemote = class(TForm)
    StatusBar: TStatusBar;
    PageControl: TPageControl;
    TabSheetRemote: TTabSheet;
    LabelComputerDir: TLabel;
    Image: TImage;
    LabelPreview: TLabel;
    ButtonTakePic: TButton;
    BitBtnClose: TBitBtn;
    ButtonStartRemoteProcess: TButton;
    TabSheetRelParams: TTabSheet;
    LabelTV: TLabel;
    LabelAv: TLabel;
    ComboBoxTv: TComboBox;
    ComboBoxAv: TComboBox;
    LabelExpComp: TLabel;
    ComboBoxExpComp: TComboBox;
    LabelFlash: TLabel;
    ComboBoxFlash: TComboBox;
    LabelFlashComp: TLabel;
    ComboBoxFlashComp: TComboBox;
    LabelSize: TLabel;
    LabelQuality: TLabel;
    ComboBoxSize: TComboBox;
    ComboBoxQuality: TComboBox;
    LabelWhiteBalance: TLabel;
    ComboBoxWhiteBalance: TComboBox;
    LabelISO: TLabel;
    ComboBoxISO: TComboBox;
    CheckBoxBeep: TCheckBox;
    LabelImageMode: TLabel;
    ComboBoxImageMode: TComboBox;
    LabelMeteringMode: TLabel;
    ComboBoxMeteringMode: TComboBox;
    LabelAFDistance: TLabel;
    ComboBoxAFDistance: TComboBox;
    LabelContrast: TLabel;
    ComboBoxContrast: TComboBox;
    LabelColorGain: TLabel;
    ComboBoxColorGain: TComboBox;
    LabelSharpness: TLabel;
    ComboBoxSharpness: TComboBox;
    ButtonGetCamSet: TButton;
    LabelZoom: TLabel;
    TrackBarZoom: TTrackBar;
    ButtonViewFinder: TButton;
    ButtonSetCameraSettings: TButton;
    ProgressBar: TProgressBar;
    LabelPhotoEffect: TLabel;
    ComboBoxPhotoEffect: TComboBox;
    LabelZoomPos: TLabel;
    TabSheetAbout: TTabSheet;
    CheckBoxDebug: TCheckBox;
    LabelVersion: TLabel;
    ComboBoxTransferMode: TComboBox;
    LabelTransfer: TLabel;
    CheckBoxOnlySupportedParam: TCheckBox;
    EditPCDirectory: TEdit;
    TimerTakePicture: TTimer;
    TimerReceivePictures: TTimer;
    CheckBoxProbeRemoteParameters: TCheckBox;
    LabelBatteryStatus: TLabel;
    LabelBatteryLevel: TLabel;
    LabelBatterySource: TLabel;
    ProgressBarBattery: TProgressBar;
    ButtonSetTime: TButton;
    EditOwnerName: TEdit;
    LabelSetOwnerName: TLabel;
    ButtonSetOwnerName: TButton;
    ButtonFormatCard: TButton;
    ButtonLockAF: TButton;
    LabelWhiteBalanceKelvin: TLabel;
    EditWhiteBalanceKelvin: TEdit;
    LabelBodyID: TLabel;
    LabelBodyIDValue: TLabel;
    LabelFirmware: TLabel;
    LabelFirmwareVersion: TLabel;
    LabelSizePic2: TLabel;
    LabelQualityPic2: TLabel;
    ComboBoxSizePic2: TComboBox;
    ComboBoxQualityPic2: TComboBox;
    ButtonPictureStyle: TButton;
    LabelDriveMode: TLabel;
    ComboBoxDriveMode: TComboBox;
    LabelAFMode: TLabel;
    ComboBoxAFMode: TComboBox;
    LabelPicturesLeft: TLabel;
    LabelColorspace: TLabel;
    ComboBoxColorSpace: TComboBox;
    ButtonWhiteBalanceShift: TButton;
    LabelAFFocusingPoint: TLabel;
    ComboBoxAFFocusingPoint: TComboBox;
    CheckBoxAFLight: TCheckBox;
    ButtonSaveParameters: TButton;
    ButtonLoadParameters: TButton;
    CamRemoteActiveX: TCamRemoteActiveXX;
    procedure FormCreate(Sender: TObject);
    procedure ButtonTakePicClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ButtonStartRemoteProcessClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ButtonGetCamSetClick(Sender: TObject);
    procedure TrackBarZoomChange(Sender: TObject);
    procedure ButtonViewFinderClick(Sender: TObject);
    procedure ButtonSetCameraSettingsClick(Sender: TObject);
    procedure CheckBoxDebugClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TimerTakePictureTimer(Sender: TObject);
    procedure TimerReceivePicturesTimer(Sender: TObject);
    procedure ButtonSetTimeClick(Sender: TObject);
    procedure ButtonSetOwnerNameClick(Sender: TObject);
    procedure ButtonFormatCardClick(Sender: TObject);
    procedure ButtonLockAFClick(Sender: TObject);
    procedure ComboBoxWhiteBalanceChange(Sender: TObject);
    procedure EditWhiteBalanceKelvinChange(Sender: TObject);
    procedure ComboBoxQualityChange(Sender: TObject);
    procedure ButtonPictureStyleClick(Sender: TObject);
    procedure ButtonWhiteBalanceShiftClick(Sender: TObject);
    procedure ButtonLoadParametersClick(Sender: TObject);
    procedure ButtonSaveParametersClick(Sender: TObject);
    procedure CamRemoteActiveXEvent(ASender: TObject; Severity,
      Event: TOleEnum);
    procedure CamRemoteActiveXRemoteEvent(ASender: TObject; Event,
      EventValid: TOleEnum; NumOfEvents: Integer; TypeOfPicture: TOleEnum);
    procedure CamRemoteActiveXRemoteGetPictureEvent(ASender: TObject;
      PercentageDone: Integer);
    procedure CamRemoteActiveXRemoteProbeParamEvent(ASender: TObject;
      PercentageDone: Integer);
    procedure CamRemoteActiveXRemoteTakePictureEvent(ASender: TObject;
      PercentageDone: Integer);
    procedure CamRemoteActiveXViewfinderEvent(ASender: TObject;
      const ViewfinderFileName: WideString);
  private
    mReleaseProcessActive  : boolean;                //Is true when the releasestart proc has been called
    mCameraIsActive        : boolean;                //Is true if the camera is ready to take a picture
    mViewFinderEnabled     : boolean;                //Is true when viewfinder functions are on
    mCameraRemoteParam     : CameraRemoteParamType;  //Contains remote parameters possible to set for current camera
    mCurrentCamera         : AvailRemoteCamerasType; //Current connected camera
    mPictureReqTime        : TDateTime;              //The time when the request for taking a picture
    mIniFile               : TRegistryIniFile;       //The registry entry
    mLogFile               : TFileStream;            //The log-file
    mConnToCamera          : boolean;                //Is true during connection to camera
    mMessageThread         : TMessageThread;         //Thread to handle event messages from the camera
    mCameraInfo            : ConnectInfoType;        //Info about the camera received when connecting
    mCameraCapability      : RemoteFuncType;         //Remote capability of the connected camera
    mReleaseMode           : ReleaseModeType;        //The selected transfer mode in "Transfer mode"
    mThumbnailsToReceive   : integer;                //The numbers of thumbnails to receive from the camera
    mPicturesToReceive     : integer;                //The numbers of pictures to receive from the camera
    mViewfinderPic         : TBitmap;                //A bitmap to store Viewfinder pictures
    mNewViewfinder         : boolean;                //Is true when a new Viewfinder pictures has been downloaded from the camera
    mCamerasEnumerated     : boolean;                //Is true when cameras have been enumerated

    procedure Initvar;
    procedure StartReleasePic;
    procedure StopReleasePic;
    procedure TakePicture;
    procedure TakeThePictures;
    procedure SetRemoteParameters(inParam : RemoteReleaseParametersType);
    function  GetRemoteParameters : RemoteReleaseParametersType;
    procedure LogRemoteParameters(RemoteParameters : RemoteReleaseParametersType);
    procedure PictureTaken;
    procedure EnableButtons(Active : boolean);
    function  Versiontext : string;
    function  GetLogFileName : string;
    procedure AppException(Sender: TObject; E: Exception);
    procedure WriteVerToLog;
    procedure DisconnectFromCamera;
    procedure ConnectToCamera;
    procedure UpdateRemoteParamVCL;
    procedure UpdateAvailableShots;
  public
    { Public declarations }
    property  ViewFinderEnabled    : boolean        read mViewFinderEnabled  write mViewFinderEnabled;
    property  MessageThread        : TMessageThread read mMessageThread      write mMessageThread;
    property  ReleaseProcessActive : boolean        read mReleaseProcessActive;
    property  NewViewfinder        : boolean        read mNewViewfinder      write mNewViewfinder;
    property  ViewfinderPic        : TBitmap        read mViewfinderPic;

    procedure Log(Text : string);
  end;

var
  FormRemote: TFormRemote;

implementation

uses DlgViewFinder, Jpeg, DlgSelectCamera, DlgPictureStyle,
  DlgWhitebalanceShift, DateUtils;

{$R *.DFM}

//Takes a picture remotely and gets the picture data if the remote transfer
//mode is active and not passive
procedure TFormRemote.TakeThePictures;
begin
  TakePicture;
end;

//Form create procedure
procedure TFormRemote.FormCreate(Sender: TObject);
begin
  //Set the application exception handler
  Application.OnException := AppException;
  //Create the registry ini-file
  mIniFile := TRegistryIniFile.Create('Software');
  CheckBoxDebug.Checked := mIniFile.ReadBool(REMOTE_SUBKEY,
                                             CheckBoxDebug.Name,
                                             false);
  //Create the debug file if the checkbox is set
  if (CheckBoxDebug.Checked) then
  begin
    if FileExists(GetLogFileName) then
    begin
      mLogFile := TFileStream.Create(GetLogFileName,
                                     (fmOpenReadWrite or fmShareDenyWrite));
      mLogFile.Seek(0, soFromEnd);
      //Check if size is exceeded
      if (mLogFile.Size > MAX_LOG_SIZE) then
      begin
        mLogFile.Free;
        //Create the new file
        mLogFile := TFileStream.Create(GetLogFileName,
                                       (fmCreate or fmShareDenyWrite));
      end;
    end else
    begin
      mLogFile := TFileStream.Create(GetLogFileName,(fmCreate or fmShareDenyWrite));
    end;
  end;
  Log('');
  Log('');
  Log('Remote template version ' + Versiontext + ' started');
  Log('Date and time=' + DateTimeToStr(Now));
  //Write OS version in LOG
  WriteVerToLog;
  //Initialize
  mConnToCamera        := false;
  StatusBar.SimpleText := '';
  //Set the global time variable format
  TimeSeparator   := ':';
  LongTimeFormat  := 'HH:mm:ss';
  //Creates the message thread. The message thread is started immidiately.
  mMessageThread  := TMessageThread.Create(false);
  mCameraIsActive := false;
  //Read data stored in the registry
  EditPCDirectory.Text := mIniFile.ReadString(REMOTE_SUBKEY,
                                              EditPCDirectory.Name,
                                              GetCurrentDir);
  CheckBoxDebug.Checked := mIniFile.ReadBool(REMOTE_SUBKEY,
                                             CheckBoxDebug.Name,
                                             CheckBoxDebug.Checked);
  ComboBoxTransferMode.ItemIndex := mIniFile.ReadInteger(REMOTE_SUBKEY,
                                                         ComboBoxTransferMode.Name,
                                                         ComboBoxTransferMode.ItemIndex);
  CheckBoxOnlySupportedParam.Checked := mIniFile.ReadBool(REMOTE_SUBKEY,
                                                          CheckBoxOnlySupportedParam.Name,
                                                          CheckBoxOnlySupportedParam.Checked);
  CheckBoxProbeRemoteParameters.Checked := mIniFile.ReadBool(REMOTE_SUBKEY,
                                                             CheckBoxProbeRemoteParameters.Name,
                                                             CheckBoxProbeRemoteParameters.Checked);
  ButtonLockAF.Caption := LOCK_AF_TXT;
  Initvar;
  //Deaktivate buttons since the remote process not started
  EnableButtons(false);
  PageControl.ActivePageIndex := 0;
  //Set about version text
  LabelVersion.Caption := 'Remote version ' + Versiontext;
  mPicturesToReceive   := 0;
  mThumbnailsToReceive := 0;
  mNewViewfinder       := false;
  mViewfinderPic       := TBitmap.Create;
  mCamerasEnumerated   := false;
  inherited;
end;

//Procedure for the "Take a picture" button
procedure TFormRemote.ButtonTakePicClick(Sender: TObject);
begin
  //Takes the picture(s) manually triggered
  Log('Enters ButtonTakePicClick');
  //Check if destination directory exists
  if not DirectoryExists(EditPCDirectory.Text) then
  begin
    raise EInOutError.Create('Destination directory does not exist');
  end;
  try
    Initvar;
    EnableButtons(false);
    StatusBar.SimpleText := 'Taking the picture';
    TakeThePictures;
    EnableButtons(true);
  except
    on E:exception do
      begin
        Log('Button_TakePicClick Exception=' + E.Message);
        Initvar;
        StopReleasePic;
        StatusBar.SimpleText := 'Remote process stopped';
        raise;
      end;
  end;
end;

//Gets the thumbnail and picture when a picture has been taken
procedure TFormRemote.PictureTaken;
var thumb_file_name   : string;
    pic_file_name     : string;
    time_string       : string;
    i                 : integer;
begin
  //Is called when the event for a remote picture is sent
  Log('Enters PictureTaken');
  try
    if ((mReleaseProcessActive) and
        (mReleaseMode <> ReleaseModeOnlyToCamera)) then
    begin
      if (mThumbnailsToReceive > 0) then
      begin
        mThumbnailsToReceive := mThumbnailsToReceive - 1;
        //First get the thumbnail
        thumb_file_name := extractFileDir(ParamStr(0)) + THUMBNAIL_FILENAME;
        Log('Remote: Thumbnail receive request. Time = ' +
            floattostr((Now - mPictureReqTime) * 24* 3600));
        CamRemoteActiveX.RemoteGetPicture(thumb_file_name,
                                          Ord(PictureTypeThumbnail));
        Log('Remote: Thumbnail received. Time = ' +
            floattostr((Now - mPictureReqTime) * 24* 3600));
        try
          Image.Picture.LoadFromFile(thumb_file_name);
        except
          on e:exception do
          begin
            //For the S3IS the thumbnail picture is corrupt. Why?
            Log('There is a problem with the thumbnail picture. Error = ' +
                e.Message);
          end;
        end
      end;
      DateTimeToString(time_string, 'yymmddhhnnss', Now);
      pic_file_name := Format('%s%s.jpg', [EditPCDirectory.Text + '\',
                                           time_string]);
      //Check if that file already exists, if so append a suffix to the filename
      if FileExists(pic_file_name) then
      begin
        i := 1;
        repeat
          pic_file_name := Format('%s%s_%d.jpg', [EditPCDirectory.Text + '\',
                                                  time_string,
                                                  i]);
          I := i + 1;
        until (not FileExists(pic_file_name));
      end;
      //Get the picure
      Log('Remote: Picture receive request. Time = ' +
          floattostr((Now - mPictureReqTime) * 24* 3600));
      CamRemoteActiveX.RemoteGetPicture(pic_file_name,
                                        Ord(PictureTypeFull));
      Log('Remote: Picture received. Time = ' +
          floattostr((Now - mPictureReqTime) * 24* 3600));
    end else begin
      Log('Exception, nearly. mReleaseProcessActive');
    end;
    ProgressBar.Position := 0;
    UpdateAvailableShots;
  except
    on E:Exception do
       begin
         Log('Exception=' + E.Message);
         Initvar;
         StopReleasePic;
         StatusBar.SimpleText := 'Remote process stopped';
         raise;
       end;
  end;
end;

procedure TFormRemote.Initvar;
begin
  Log('Enters Initvar');
  if (ButtonStartRemoteProcess.Caption = STOP_REMOTE_PROC_TXT) then
  begin
    ButtonTakePic.Enabled := true;
  end;
  ButtonTakePic.Enabled;
end;

//Procedure used when starting the remote process
procedure TFormRemote.StartReleasePic;
var i              : integer;
    temp_tab_sheet : TTabSheet;

  procedure set_remote_array(var remote_array : array of boolean; remote_string : string);
  var i : integer;
  begin
    for i := 0 to Length(remote_string) - 1 do
    begin
      remote_array[i] := false;
      if (remote_string[i + 1] = '1') then
      begin
        remote_array[i] := true;
      end;
    end;
  end;

begin
  Log('Enters StartReleasePic');
  Log('Checks if the camera supports remote');
  if (not CamRemoteActiveX.RemoteSupported) then
  begin
    ECamException.Create('The connected camera does not support remote mode');
  end;
  StatusBar.SimpleText := 'Starts remote process and probes remote parameters';
  //Starts the remote mode
  Log('Opens remote mode');
{$IFNDEF VER130} //Not supported by Delphi 5
  CamRemoteActiveX.RemoteStart(Ord(mReleaseMode),
                               Ord(REMOTE_MODE),
                               CheckBoxProbeRemoteParameters.Checked,
                               GetEnvironmentVariable('Temp') + '\' + REMOTE_SUBKEY);
{$ELSE}
  CamRemoteActiveX.RemoteStart(Ord(mReleaseMode),
                               Ord(REMOTE_MODE),
                               CheckBoxProbeRemoteParameters.Checked,
                               ExtractFileDir(ParamStr(0)));
{$ENDIF}
{$IFNDEF VER130} //Not supported by Delphi 5
  mCameraCapability.DoSupportZoom                := CamRemoteActiveX.RemoteStart_DoSupportZoom;
  mCameraCapability.DoSupportShootingPara        := CamRemoteActiveX.RemoteStart_DoSupportShootingPara;
  mCameraCapability.DoSupportViewfinder          := CamRemoteActiveX.RemoteStart_DoSupportViewfinder;
  mCameraCapability.ReqViewfinderOffWhenShooting := CamRemoteActiveX.RemoteStart_ReqViewfinderOffWhenShooting;
  mCameraCapability.DoSupportAfLockUnlock        := CamRemoteActiveX.RemoteStart_DoSupportAfLockUnlock;

  set_remote_array(mCameraCapability.RemoteParamSupported.ImageSize,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamImageSize));
  set_remote_array(mCameraCapability.RemoteParamSupported.CompQuality,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamCompQuality));
  set_remote_array(mCameraCapability.RemoteParamSupported.ImageSizePic2RAW,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamImageSizePic2RAW));
  set_remote_array(mCameraCapability.RemoteParamSupported.CompQualityPic2RAW,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamCompQualityPic2RAW));
  set_remote_array(mCameraCapability.RemoteParamSupported.StrobeSetting,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamStrobeSetting));
  set_remote_array(mCameraCapability.RemoteParamSupported.StrobeCompSetting,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamStrobeCompSetting));
  set_remote_array(mCameraCapability.RemoteParamSupported.DriveMode,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamDriveMode));
  set_remote_array(mCameraCapability.RemoteParamSupported.ImageMode,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamImageMode));
  set_remote_array(mCameraCapability.RemoteParamSupported.MLWeiMode,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamMLWeiMode));
  set_remote_array(mCameraCapability.RemoteParamSupported.AFMode,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamAFMode));
  set_remote_array(mCameraCapability.RemoteParamSupported.AFDistance,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamAFDistance));
  set_remote_array(mCameraCapability.RemoteParamSupported.AFFocusingPoint,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamAFFocusingPoint));
  set_remote_array(mCameraCapability.RemoteParamSupported.AFAssistLight,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamAFAssistLight));
  set_remote_array(mCameraCapability.RemoteParamSupported.WhiteBalanceSetting,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamWhiteBalanceSetting));
  if (CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamWhiteBalanceShift) = '1') then
  begin
    mCameraCapability.RemoteParamSupported.WhiteBalanceShift := true;
  end;
  if (CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamPictureStyle) = '1') then
  begin
    mCameraCapability.RemoteParamSupported.PictureStyle := true;
  end;
  set_remote_array(mCameraCapability.RemoteParamSupported.Contrast,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamContrast));
  set_remote_array(mCameraCapability.RemoteParamSupported.ColorGain,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamColorGain));
  set_remote_array(mCameraCapability.RemoteParamSupported.Sharpness,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamSharpness));
  set_remote_array(mCameraCapability.RemoteParamSupported.ColorSpace,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamColorSpace));
  set_remote_array(mCameraCapability.RemoteParamSupported.ISO,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamISO));
  set_remote_array(mCameraCapability.RemoteParamSupported.Av,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamAv));
  set_remote_array(mCameraCapability.RemoteParamSupported.Tv,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamTv));
  set_remote_array(mCameraCapability.RemoteParamSupported.ExposureCompensation,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamExposureCompensation));
  set_remote_array(mCameraCapability.RemoteParamSupported.PhotoEffect,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamPhotoEffect));
  set_remote_array(mCameraCapability.RemoteParamSupported.Beep,
                   CamRemoteActiveX.RemoteStart_GetRemoteParamSupported(RemoteParamBeep));
  Log('Remote capabaility DoSupportZoom = '                + BoolToStr(mCameraCapability.DoSupportZoom));
  Log('Remote capabaility DoSupportShootingPara = '        + BoolToStr(mCameraCapability.DoSupportShootingPara));
  Log('Remote capabaility DoSupportViewfinder = '          + BoolToStr(mCameraCapability.DoSupportViewfinder));
  Log('Remote capabaility ReqViewfinderOffWhenShooting = ' + BoolToStr(mCameraCapability.ReqViewfinderOffWhenShooting));
  Log('Remote capabaility DoSupportAfLockUnlock = '        + BoolToStr(mCameraCapability.DoSupportAfLockUnlock));
{$ENDIF}
  mCameraIsActive       := true;
  mReleaseProcessActive := true;
  //Disable not supported VCL
  if (not mCameraCapability.DoSupportViewfinder) then
  begin
    ButtonViewFinder.Visible := false;
    Log('ViewFinder is not allowed');
  end else begin
    ButtonViewFinder.Enabled := true;
    Log('ViewFinder is allowed');
  end;
  if (not mCameraCapability.DoSupportAfLockUnlock) then
  begin
    ButtonLockAF.Visible := false;
    Log('AFLock is not allowed');
  end else begin
    ButtonLockAF.Enabled := true;
    Log('AFLock is allowed');
  end;
  if (not mCameraCapability.DoSupportZoom) then
  begin
    LabelZoom.Visible    := false;
    TrackBarZoom.Visible := false;
  end;
  if (not mCameraCapability.DoSupportShootingPara) then
  begin
    //Disable the parameters tab
    for i := 0 to PageControl.PageCount - 1 do
    begin
       temp_tab_sheet := PageControl.Pages[i];
       if (temp_tab_sheet.Name = 'TabSheetRelParams') then
       begin
         temp_tab_sheet.TabVisible := false;
       end;
    end;
  end;
  EnableButtons(true);
  StatusBar.SimpleText := 'Remote process is opened';
end;

//Procedure used when stopping the remote process
procedure TFormRemote.StopReleasePic;
begin
  //Ends the release process
  Log('Enters StopReleasePic');
  if (mReleaseProcessActive) then
  begin
    EnableButtons(false);
    ButtonStartRemoteProcess.Caption := START_REMOTE_PROC_TXT;
    CamRemoteActiveX.RemoteEnd;
    mCameraIsActive := false;
    Log('StopReleasePic, ReleaseEnd');
    mReleaseProcessActive := false;
  end;
  StatusBar.SimpleText := 'Remote process stopped';
  Initvar;
end;

//Procedure used when taking a picture remotely
procedure TFormRemote.TakePicture;
//var remote_parameters : RemoteReleaseParametersType;
begin
  //Takes one picture
  Log('Enters TakePicture');
  mPictureReqTime := Now();
  Log('Remote: Request to take a picture. Time = ' +
      floattostr((Now - mPictureReqTime) * 24* 3600));
  //Optionally set the remote parameters... Mayve even turn off the viewfinder
  //before setting these parameters (see the nahial for more information)
  //if (mCameraCapability.DoSupportShootingPara) then
  //begin
    ////Gets release parameters from the Parameters tab
    //remote_parameters := GetRemoteParameters;
    //Log('Sets remote parameters');
    //CamRemote.RemoteSetRemoteParams(remote_parameters);
    //Log('Remote: Remote parameters set. Time = ' +
    //    floattostr((Now - mPictureReqTime) * 24* 3600));
  //end;
  Log('Takes a remote pic. Zoom = ' + inttostr(TrackBarZoom.Position)+
      ', maxZoom = ' + inttostr(TrackBarZoom.Max));
  //Check if Viewfinder must be closed before taking the picture
  if (FormViewFinder.Showing) and (mCameraCapability.ReqViewfinderOffWhenShooting) then
  begin
    FormViewFinder.Close;
  end;
  CamRemoteActiveX.RemoteTakePicture;
  Log('Remote: Request for taking picture done. Time = ' +
      floattostr((Now - mPictureReqTime) * 24* 3600));
end;

//Procedure called when exiting the application and the form is freed.
procedure TFormRemote.FormDestroy(Sender: TObject);
begin
  Log('Enters TForm_Remote.FormDestroy');
  mViewfinderPic.Free;
  //Stores data in the registry
  mIniFile.WriteInteger(REMOTE_SUBKEY, 'RemoteWidth',                      FormRemote.Width);
  mIniFile.WriteInteger(REMOTE_SUBKEY, 'RemoteHeight',                     FormRemote.Height);
  mIniFile.WriteString (REMOTE_SUBKEY, EditPCDirectory.Name,               EditPCDirectory.Text);
  mIniFile.WriteBool   (REMOTE_SUBKEY, CheckBoxDebug.Name,                 CheckBoxDebug.Checked);
  mIniFile.WriteInteger(REMOTE_SUBKEY, ComboBoxTransferMode.Name,          ComboBoxTransferMode.ItemIndex);
  mIniFile.WriteBool   (REMOTE_SUBKEY, CheckBoxOnlySupportedParam.Name,    CheckBoxOnlySupportedParam.Checked);
  mIniFile.WriteBool   (REMOTE_SUBKEY, CheckBoxProbeRemoteParameters.Name, CheckBoxProbeRemoteParameters.Checked);
  mIniFile.Free;
  if (CheckBoxDebug.Checked) then
  begin
    mLogFile.Free;
  end;
  //Close the message thread
  mMessageThread.Terminate;
  mMessageThread.WaitFor;
  mMessageThread.free;
end;

//EnableButtons enables or disables buttons that only can be used when remote
//process has been started or stopped.
procedure TFormRemote.EnableButtons(Active : boolean);
begin
  Log('Enters TFormRemote.EnableButtons');
  ButtonTakePic.Enabled           := Active;
  ButtonGetCamSet.Enabled         := Active;
  ButtonSetCameraSettings.Enabled := Active;
  ButtonViewFinder.Enabled        := Active;
  ButtonSetTime.Enabled           := Active;
  ButtonSetOwnerName.Enabled      := Active;
  ButtonFormatCard.Enabled        := Active;
  ButtonLockAF.Enabled            := Active;
  ButtonLoadParameters.Enabled    := Active;
  ButtonSaveParameters.Enabled    := Active;
end;

procedure TFormRemote.ButtonStartRemoteProcessClick(Sender: TObject);
begin
  //Starts the remote process
  Log('Enters TForm_Remote.Button_StartRemoteProcessClick');
  if (ButtonStartRemoteProcess.Caption = START_REMOTE_PROC_TXT) then
  begin
    //Connects to the camera and starts the remote mode
    if (ComboBoxTransferMode.ItemIndex <> -1) then
    begin
      mReleaseMode := ReleaseModeType(ComboBoxTransferMode.ItemIndex);
    end;
    ButtonStartRemoteProcess.Caption      := STOP_REMOTE_PROC_TXT;
    ComboBoxTransferMode.Enabled          := false;
    CheckBoxOnlySupportedParam.Enabled    := false;
    CheckBoxProbeRemoteParameters.Enabled := false;
    ConnectToCamera;
    if (CheckBoxOnlySupportedParam.Checked) then
    begin
      mCurrentCamera := ConnectedCameraParameters;
    end else begin
      mCurrentCamera := AllParameters;
    end;
    StartReleasePic;
    UpdateRemoteParamVCL;
    UpdateAvailableShots;
  end else
  begin
    //Closes the viewfinder, if opened, then stops the remote mode and
    //disconnects from the camera
    if (FormViewFinder.Visible) then
    begin
      FormViewFinder.Close;
    end;
    //Disconnects from the camera
    ButtonStartRemoteProcess.Caption     := START_REMOTE_PROC_TXT;
    ComboBoxTransferMode.Enabled         := true;
    CheckBoxOnlySupportedParam.Enabled   := true;
    CheckBoxProbeRemoteParameters.Enabled := true;
    try
      StopReleasePic;
    finally
      DisconnectFromCamera;
    end;
  end;
end;

procedure TFormRemote.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  //Closes the viewfinder, if opened, then stops the remote mode and
  //disconnects from the camera
  Log('Enters TForm_Remote.FormClose');
  if (FormViewFinder.Visible) then
  begin
    FormViewFinder.Close;
  end;
  if mCameraIsActive then
  begin
    StopReleasePic;
    DisconnectFromCamera;
  end;
  if mCamerasEnumerated then
  begin
    CamRemoteActiveX.CloseCameraEnumeration;
  end;
end;

procedure TFormRemote.ButtonGetCamSetClick(Sender: TObject);
var remote_param    : RemoteReleaseParametersType;
    zoom_capability : RemoteZoomCapabilityType;
begin
  //Clear local variable
  FillChar(remote_param, sizeof(RemoteReleaseParametersType), #0);
  //Gets remote parameters from the camera and sets the VCL components
  remote_param.CompQuality :=
    RemoteFormatQualityType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamCompQuality));
  remote_param.CompQualityPic2RAW :=
    RemoteFormatQualityType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamCompQualityPic2RAW));
  remote_param.ImageSize :=
    RemoteFormatSizeType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamImageSize));
  remote_param.ImageSizePic2RAW :=
    RemoteFormatSizeType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamImageSizePic2RAW));
  remote_param.StrobeSetting :=
    RemoteFormatFlashType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamStrobeSetting));
  remote_param.StrobeCompSetting :=
    RemoteFormatFlashCompType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamStrobeCompSetting));
  remote_param.DriveMode :=
    RemoteFormatDriveModeType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamDriveMode));
  remote_param.ImageMode :=
    RemoteFormatShootingModeType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamImageMode));
  remote_param.MLWeiMode :=
    RemoteFormatMLWeiType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamMLWeiMode));
  remote_param.AFMode :=
    RemoteFormatAFModeType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamAFMode));
  remote_param.AFDistance :=
    RemoteFormatAFDistType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamAFDistance));
  remote_param.AFFocusingPoint :=
    RemoteFormatAFFocusingPointType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamAFFocusingPoint));
  remote_param.AFAssistLight :=
    RemoteFormatAFLightType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamAFAssistLight));
  remote_param.WhiteBalanceSetting :=
    RemoteFormatWBType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamWhiteBalanceSetting));
  remote_param.WhiteBalanceKelvin :=
    CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamWhiteBalanceKelvin);
  remote_param.WhiteBalanceShift.WBShiftEnabled :=
    Boolean(CamRemoteActiveX.RemoteGetRemoteParams_WhiteBalanceShift(RemoteParamWhitebalanceShiftWBShiftEnabled));
  remote_param.WhiteBalanceShift.AmberBlue :=
    CamRemoteActiveX.RemoteGetRemoteParams_WhiteBalanceShift(RemoteParamWhitebalanceShiftAmberBlue);
  remote_param.WhiteBalanceShift.GreenMagenta :=
    CamRemoteActiveX.RemoteGetRemoteParams_WhiteBalanceShift(RemoteParamWhitebalanceShiftGreenMagenta);
  remote_param.PictureStyle.PictureStyleEnabled :=
    Boolean(CamRemoteActiveX.RemoteGetRemoteParams_PictureStyle(RemoteParamPictureStyleEnabled));
  remote_param.PictureStyle.Contrast :=
    CamRemoteActiveX.RemoteGetRemoteParams_PictureStyle(RemoteParamPictureStyleContrast);
  remote_param.PictureStyle.Sharpness :=
    CamRemoteActiveX.RemoteGetRemoteParams_PictureStyle(RemoteParamPictureStyleSharpness);
  remote_param.PictureStyle.Saturation :=
    CamRemoteActiveX.RemoteGetRemoteParams_PictureStyle(RemoteParamPictureStyleSaturation);
  remote_param.PictureStyle.SaturationUsed :=
    Boolean(CamRemoteActiveX.RemoteGetRemoteParams_PictureStyle(RemoteParamPictureStyleSaturationUsed));
  remote_param.PictureStyle.ColorTone :=
    CamRemoteActiveX.RemoteGetRemoteParams_PictureStyle(RemoteParamPictureStyleColorTone);
  remote_param.PictureStyle.ColorToneUsed :=
    Boolean(CamRemoteActiveX.RemoteGetRemoteParams_PictureStyle(RemoteParamPictureStyleColorToneUsed));
  remote_param.PictureStyle.MonochromeFilter :=
    MonochromeFilterType(CamRemoteActiveX.RemoteGetRemoteParams_PictureStyle(RemoteParamPictureStyleMonochromeFilter));
  remote_param.PictureStyle.MonochromeFilterUsed :=
    Boolean(CamRemoteActiveX.RemoteGetRemoteParams_PictureStyle(RemoteParamPictureStyleMonochromeFilterUsed));
  remote_param.PictureStyle.MonochromeTone :=
    MonochromeToneType(CamRemoteActiveX.RemoteGetRemoteParams_PictureStyle(RemoteParamPictureStyleMonochromeTone));
  remote_param.PictureStyle.MonochromeToneUsed :=
    Boolean(CamRemoteActiveX.RemoteGetRemoteParams_PictureStyle(RemoteParamPictureStyleMonochromeToneUsed));
  remote_param.Contrast :=
    RemoteFormatLevelType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamContrast));
  remote_param.ColorGain :=
    RemoteFormatLevelType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamColorGain));
  remote_param.Sharpness :=
     RemoteFormatLevelType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamSharpness));
  remote_param.ColorSpace :=
    RemoteFormatColorSpaceType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamColorSpace));
  remote_param.ISO :=
    RemoteFormatISOType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamISO));
  remote_param.Av :=
    RemoteFormatAVType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamAv));
  remote_param.Tv :=
    RemoteFormatTVType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamTv));
  remote_param.ExposureCompensation :=
    RemoteFormatExposureCompType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamExposureCompensation));
  remote_param.PhotoEffect :=
    RemoteFormatPhotoEffectType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamPhotoEffect));
  remote_param.Beep :=
    RemoteFormatBeepType(CamRemoteActiveX.RemoteGetRemoteParams(RemoteParamBeep));
  Log('Sets remote parameters to dialogue');
  LogRemoteParameters(remote_param);
  SetRemoteParameters(remote_param);
  ComboBoxWhiteBalanceChange(Sender); //Update WB combobox to enable Kelvin settings
  ComboBoxQualityChange(Sender);      //Update pic 2 quality if RAW is used
  if (mCameraCapability.DoSupportZoom) then begin
    //Get zoom position
    zoom_capability.CurrentZoomPos    := CamRemoteActiveX.RemoteGetZoomPos_CurrentZoomPos;
    zoom_capability.MaxOpticalZoomPos := CamRemoteActiveX.RemoteGetZoomPos_MaxOpticalZoomPos;
    zoom_capability.MaxZoomPos        := CamRemoteActiveX.RemoteGetZoomPos_MaxZoomPos;
    TrackBarZoom.Max                  := zoom_capability.MaxOpticalZoomPos;
    TrackBarZoom.Position             := zoom_capability.CurrentZoomPos;
    LabelZoomPos.Caption              := IntToStr(TrackBarZoom.Position);
    TrackBarZoom.Frequency            := TrackBarZoom.Max DIV 10;
  end;
end;

procedure TFormRemote.FormShow(Sender: TObject);
begin
  //Resize dialogue
  FormRemote.Width  := mIniFile.ReadInteger(REMOTE_SUBKEY, 'RemoteWidth',  FormRemote.Width);
  FormRemote.Height := mIniFile.ReadInteger(REMOTE_SUBKEY, 'RemoteHeight', FormRemote.Height);
  //Set current camera
  Initvar;
  CamRemoteActiveX.Visible := false;
end;

//Updates the comboboxes in the remote tab
procedure TFormRemote.UpdateRemoteParamVCL;
var i                           : integer;
    quality_counter             : RemoteFormatQualityType;
    imagesize_counter           : RemoteFormatSizeType;
    strobesetting_counter       : RemoteFormatFlashType;
    imagemode_counter           : RemoteFormatShootingModeType;
    drivemode_counter           : RemoteFormatDriveModeType;
    mlweimode_counter           : RemoteFormatMLWeiType;
    afmode_counter              : RemoteFormatAFModeType;
    afdistance_counter          : RemoteFormatAFDistType;
    affocusingpoint_counter     : RemoteFormatAFFocusingPointType;
    afassist_light_counter      : RemoteFormatAFLightType;
    whitebalancesetting_counter : RemoteFormatWBType;
    colorspace_counter          : RemoteFormatColorSpaceType;
    iso_counter                 : RemoteFormatISOType;
    expcomp_counter             : RemoteFormatExposureCompType;
    av_counter                  : RemoteFormatAVType;
    tv_counter                  : RemoteFormatTVType;
    photoeffect_counter         : RemoteFormatPhotoEffectType;
    strobecomp_counter          : RemoteFormatFlashCompType;
    level_counter               : RemoteFormatLevelType;
    beep_counter                : RemoteFormatBeepType;
begin
  //Assign camera parameters possible to set
  //CompQuality
  i := 0;
  SetLength(mCameraRemoteParam.CompQuality, 0);
  for quality_counter := Succ(Low(RemoteFormatQualityType)) to High(RemoteFormatQualityType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.CompQuality[quality_counter] = true)) or
        (mCurrentCamera = AllParameters)) then
    begin
      SetLength(mCameraRemoteParam.CompQuality, i + 1);
      mCameraRemoteParam.CompQuality[i] := quality_counter;
      i := i + 1;
    end;
  end;

  //CompQualityPic2RAW
  i := 0;
  SetLength(mCameraRemoteParam.CompQualityPic2RAW, 0);
  for quality_counter := Succ(Low(RemoteFormatQualityType)) to High(RemoteFormatQualityType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.CompQualityPic2RAW[quality_counter] = true)) or
        (mCurrentCamera = AllParameters)) then
    begin
      SetLength(mCameraRemoteParam.CompQualityPic2RAW, i + 1);
      mCameraRemoteParam.CompQualityPic2RAW[i] := quality_counter;
      i := i + 1;
    end;
  end;

  //Image size
  i := 0;
  SetLength(mCameraRemoteParam.ImageSize, 0);
  for imagesize_counter := Succ(Low(RemoteFormatSizeType)) to High(RemoteFormatSizeType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.ImageSize[imagesize_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.ImageSize, i + 1);
       mCameraRemoteParam.ImageSize[i] := imagesize_counter;
       i := i + 1;
    end;
  end;

  //Image size Pic2RAW
  i := 0;
  SetLength(mCameraRemoteParam.ImageSizePic2RAW, 0);
  for imagesize_counter := Succ(Low(RemoteFormatSizeType)) to High(RemoteFormatSizeType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.ImageSizePic2RAW[imagesize_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.ImageSizePic2RAW, i + 1);
       mCameraRemoteParam.ImageSizePic2RAW[i] := imagesize_counter;
       i := i + 1;
    end;
  end;

  //StrobeSetting
  i := 0;
  SetLength(mCameraRemoteParam.StrobeSetting, 0);
  for strobesetting_counter := Succ(Low(RemoteFormatFlashType)) to High(RemoteFormatFlashType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.StrobeSetting[strobesetting_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.StrobeSetting, i + 1);
       mCameraRemoteParam.StrobeSetting[i] := strobesetting_counter;
       i := i + 1;
    end;
  end;

  //StrobeCompSetting
  i := 0;
  SetLength(mCameraRemoteParam.StrobeCompSetting, 0);
  for strobecomp_counter := Succ(Low(RemoteFormatFlashCompType)) to High(RemoteFormatFlashCompType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.StrobeCompSetting[strobecomp_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.StrobeCompSetting, i + 1);
       mCameraRemoteParam.StrobeCompSetting[i] := strobecomp_counter;
       i := i + 1;
    end;
  end;

  //ImageMode
  i := 0;
  SetLength(mCameraRemoteParam.ImageMode, 0);
  for imagemode_counter := Succ(Low(RemoteFormatShootingModeType)) to High(RemoteFormatShootingModeType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.ImageMode[imagemode_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.ImageMode, i + 1);
       mCameraRemoteParam.ImageMode[i] := imagemode_counter;
       i := i + 1;
    end;
  end;

  //DriveMode
  i := 0;
  SetLength(mCameraRemoteParam.DriveMode, 0);
  for drivemode_counter := Succ(Low(RemoteFormatDriveModeType)) to High(RemoteFormatDriveModeType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.DriveMode[drivemode_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.DriveMode, i + 1);
       mCameraRemoteParam.DriveMode[i] := drivemode_counter;
       i := i + 1;
    end;
  end;

  //MLWeiMode
  i := 0;
  SetLength(mCameraRemoteParam.MLWeiMode, 0);
  for mlweimode_counter := Succ(Low(RemoteFormatMLWeiType)) to High(RemoteFormatMLWeiType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.MLWeiMode[mlweimode_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.MLWeiMode, i + 1);
       mCameraRemoteParam.MLWeiMode[i] := mlweimode_counter;
       i := i + 1;
    end;
  end;

  //AFMode
  i := 0;
  SetLength(mCameraRemoteParam.AFMode, 0);
  for afmode_counter := Succ(Low(RemoteFormatAFModeType)) to High(RemoteFormatAFModeType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.AFMode[afmode_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.AFMode, i + 1);
       mCameraRemoteParam.AFMode[i] := afmode_counter;
       i := i + 1;
    end;
  end;

  //AFDistance
  i := 0;
  SetLength(mCameraRemoteParam.AFDistance, 0);
  for afdistance_counter := Succ(Low(RemoteFormatAFDistType)) to High(RemoteFormatAFDistType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.AFDistance[afdistance_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.AFDistance, i + 1);
       mCameraRemoteParam.AFDistance[i] := afdistance_counter;
       i := i + 1;
    end;
  end;

  //AFFocusingPoint
  i := 0;
  SetLength(mCameraRemoteParam.AFFocusingPoint, 0);
  for affocusingpoint_counter := Succ(Low(RemoteFormatAFFocusingPointType)) to High(RemoteFormatAFFocusingPointType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.AFFocusingPoint[affocusingpoint_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.AFFocusingPoint, i + 1);
       mCameraRemoteParam.AFFocusingPoint[i] := affocusingpoint_counter;
       i := i + 1;
    end;
  end;

  //AFAssistLight
  i := 0;
  SetLength(mCameraRemoteParam.AFAssistLight, 0);
  for afassist_light_counter := Succ(Low(RemoteFormatAFLightType)) to High(RemoteFormatAFLightType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.AFAssistLight[afassist_light_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.AFAssistLight, i + 1);
       mCameraRemoteParam.AFAssistLight[i] := afassist_light_counter;
       i := i + 1;
    end;
  end;

  //WhiteBalanceSetting
  i := 0;
  SetLength(mCameraRemoteParam.WhiteBalanceSetting, 0);
  for whitebalancesetting_counter := Succ(Low(RemoteFormatWBType)) to High(RemoteFormatWBType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.WhiteBalanceSetting[whitebalancesetting_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.WhiteBalanceSetting, i + 1);
       mCameraRemoteParam.WhiteBalanceSetting[i] := whitebalancesetting_counter;
       i := i + 1;
    end;
  end;

  //Contrast
  i := 0;
  SetLength(mCameraRemoteParam.Contrast, 0);
  for level_counter := Succ(Low(RemoteFormatLevelType)) to High(RemoteFormatLevelType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.Contrast[level_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.Contrast, i + 1);
       mCameraRemoteParam.Contrast[i] := level_counter;
       i := i + 1;
    end;
  end;

  //ColorGain
  i := 0;
  SetLength(mCameraRemoteParam.ColorGain, 0);
  for level_counter := Succ(Low(RemoteFormatLevelType)) to High(RemoteFormatLevelType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.ColorGain[level_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.ColorGain, i + 1);
       mCameraRemoteParam.ColorGain[i] := level_counter;
       i := i + 1;
    end;
  end;

  //Sharpness
  i := 0;
  SetLength(mCameraRemoteParam.Sharpness, 0);
  for level_counter := Succ(Low(RemoteFormatLevelType)) to High(RemoteFormatLevelType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.Sharpness[level_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.Sharpness, i + 1);
       mCameraRemoteParam.Sharpness[i] := level_counter;
       i := i + 1;
    end;
  end;

  //ColorSpace
  i := 0;
  SetLength(mCameraRemoteParam.ColorSpace, 0);
  for colorspace_counter := Succ(Low(RemoteFormatColorSpaceType)) to High(RemoteFormatColorSpaceType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.ColorSpace[colorspace_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.ColorSpace, i + 1);
       mCameraRemoteParam.ColorSpace[i] := colorspace_counter;
       i := i + 1;
    end;
  end;

  //ISO
  i := 0;
  SetLength(mCameraRemoteParam.ISO, 0);
  for iso_counter := Succ(Low(RemoteFormatISOType)) to High(RemoteFormatISOType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.ISO[iso_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.ISO, i + 1);
       mCameraRemoteParam.ISO[i] := iso_counter;
       i := i + 1;
    end;
  end;

  //ExposureComp
  i := 0;
  SetLength(mCameraRemoteParam.ExposureComp, 0);
  for expcomp_counter := Succ(Low(RemoteFormatExposureCompType)) to High(RemoteFormatExposureCompType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.ExposureCompensation[expcomp_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.ExposureComp, i + 1);
       mCameraRemoteParam.ExposureComp[i] := expcomp_counter;
       i := i + 1;
    end;
  end;

  //AV
  i := 0;
  SetLength(mCameraRemoteParam.AV, 0);
  for AV_counter := Succ(Low(RemoteFormatAVType)) to High(RemoteFormatAVType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.Av[AV_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.AV, i + 1);
       mCameraRemoteParam.AV[i] := AV_counter;
       i := i + 1;
    end;
  end;

  //TV
  i := 0;
  SetLength(mCameraRemoteParam.TV, 0);
  for TV_counter := Succ(Low(RemoteFormatTVType)) to High(RemoteFormatTVType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.Tv[TV_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.TV, i + 1);
       mCameraRemoteParam.TV[i] := TV_counter;
       i := i + 1;
    end;
  end;

  //PhotoEffect
  i := 0;
  SetLength(mCameraRemoteParam.PhotoEffect, 0);
  for photoeffect_counter := Succ(Low(RemoteFormatPhotoEffectType)) to High(RemoteFormatPhotoEffectType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.PhotoEffect[photoeffect_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.PhotoEffect, i + 1);
       mCameraRemoteParam.PhotoEffect[i] := photoeffect_counter;
       i := i + 1;
    end;
  end;

  //Beep
  i := 0;
  SetLength(mCameraRemoteParam.Beep, 0);
  for beep_counter := Succ(Low(RemoteFormatBeepType)) to High(RemoteFormatBeepType) do
  begin
    if (((mCurrentCamera = ConnectedCameraParameters) and (mCameraCapability.RemoteParamSupported.Beep[beep_counter])) or
        (mCurrentCamera = AllParameters)) then
    begin
       SetLength(mCameraRemoteParam.Beep, i + 1);
       mCameraRemoteParam.Beep[i] := beep_counter;
       i := i + 1;
    end;
  end;

  //Get the items in the parameters comboboxes
  //Compression quality combobox
  ComboBoxQuality.Enabled := false;
  ComboBoxQuality.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.CompQuality) do begin
    ComboBoxQuality.Enabled := true;
    ComboBoxQuality.Items.Add(gCompQualArrayText[mCameraRemoteParam.CompQuality[i]]);
  end;

  //Compression quality combobox
  ComboBoxQualityPic2.Enabled := false;
  ComboBoxQualityPic2.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.CompQualityPic2RAW) do begin
    ComboBoxQualityPic2.Enabled := true;
    ComboBoxQualityPic2.Items.Add(gCompQualArrayText[mCameraRemoteParam.CompQualityPic2RAW[i]]);
  end;

  //ImageSize combobox
  ComboBoxSize.Enabled := false;
  ComboBoxSize.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.ImageSize) do begin
    ComboBoxSize.Enabled := true;
    ComboBoxSize.Items.Add(gImageSizeArrayText[mCameraRemoteParam.ImageSize[i]]);
  end;

  //ImageSizePic2RAW combobox
  ComboBoxSizePic2.Enabled := false;
  ComboBoxSizePic2.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.ImageSizePic2RAW) do begin
    ComboBoxSizePic2.Enabled := true;
    ComboBoxSizePic2.Items.Add(gImageSizeArrayText[mCameraRemoteParam.ImageSizePic2RAW[i]]);
  end;

  //StrobeSetting combobox
  ComboBoxFlash.Enabled := false;
  ComboBoxFlash.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.StrobeSetting) do begin
    ComboBoxFlash.Enabled := true;
    ComboBoxFlash.Items.Add(gStrobeSettingArrayText[mCameraRemoteParam.StrobeSetting[i]]);
  end;

  //StrobeCompSetting combobox
  ComboBoxFlashComp.Enabled := false;
  ComboBoxFlashComp.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.StrobeCompSetting) do begin
    ComboBoxFlashComp.Enabled := true;
    ComboBoxFlashComp.Items.Add(gStrobeCompSettingArrayText[mCameraRemoteParam.StrobeCompSetting[i]]);
  end;

  //ImageMode combobox
  ComboBoxImageMode.Enabled := false;
  ComboBoxImageMode.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.ImageMode) do begin
    ComboBoxImageMode.Enabled := true;
    ComboBoxImageMode.Items.Add(gImageModeArrayText[mCameraRemoteParam.ImageMode[i]]);
  end;

  //DriveMode combobox
  ComboBoxDriveMode.Enabled := false;
  ComboBoxDriveMode.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.DriveMode) do begin
    ComboBoxDriveMode.Enabled := true;
    ComboBoxDriveMode.Items.Add(gDriveModeArrayText[mCameraRemoteParam.DriveMode[i]]);
  end;

  //MLWeiMode combobox
  ComboBoxMeteringMode.Enabled := false;
  ComboBoxMeteringMode.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.MLWeiMode) do begin
    ComboBoxMeteringMode.Enabled := true;
    ComboBoxMeteringMode.Items.Add(gMLWeiModeArrayText[mCameraRemoteParam.MLWeiMode[i]]);
  end;

  //AFMode combobox
  ComboBoxAFMode.Enabled := false;
  ComboBoxAFMode.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.AFMode) do begin
    ComboBoxAFMode.Enabled := true;
    ComboBoxAFMode.Items.Add(gAFModeArrayText[mCameraRemoteParam.AFMode[i]]);
  end;

  //AFDistance combobox
  ComboBoxAFDistance.Enabled := false;
  ComboBoxAFDistance.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.AFDistance) do begin
    ComboBoxAFDistance.Enabled := true;
    ComboBoxAFDistance.Items.Add(gAFDistanceArrayText[mCameraRemoteParam.AFDistance[i]]);
  end;

  //AFFocusingPoint combobox
  ComboBoxAFFocusingPoint.Enabled := false;
  ComboBoxAFFocusingPoint.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.AFFocusingPoint) do begin
    ComboBoxAFFocusingPoint.Enabled := true;
    ComboBoxAFFocusingPoint.Items.Add(gAFFocusingPointArrayText[mCameraRemoteParam.AFFocusingPoint[i]]);
  end;

  //AFAssistLight checkbox
  CheckBoxAFLight.Enabled := false;
  if (high(mCameraRemoteParam.AFAssistLight) > 0) then begin
    CheckBoxAFLight.Enabled := true;
  end;

  //WhiteBalanceSetting combobox
  ComboBoxWhiteBalance.Enabled := false;
  ComboBoxWhiteBalance.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.WhiteBalanceSetting) do begin
    ComboBoxWhiteBalance.Enabled := true;
    ComboBoxWhiteBalance.Items.Add(gWhiteBalanceSetArrayText[mCameraRemoteParam.WhiteBalanceSetting[i]]);
  end;

  //Photo Effect combobox
  ComboBoxPhotoEffect.Enabled := false;
  ComboBoxPhotoEffect.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.PhotoEffect) do begin
    ComboBoxPhotoEffect.Enabled := true;
    ComboBoxPhotoEffect.Items.Add(gPhotoEffectArrayText[mCameraRemoteParam.PhotoEffect[i]]);
  end;

  //Contrast combobox
  ComboBoxContrast.Enabled := false;
  ComboBoxContrast.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.Contrast) do begin
    ComboBoxContrast.Enabled := true;
    ComboBoxContrast.Items.Add(gContrastArrayText[mCameraRemoteParam.Contrast[i]]);
  end;

  //Colorgain combobox
  ComboBoxColorGain.Enabled := false;
  ComboBoxColorGain.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.Colorgain) do begin
    ComboBoxColorGain.Enabled := true;
    ComboBoxColorGain.Items.Add(gColorGainArrayText[mCameraRemoteParam.Colorgain[i]]);
  end;

  //Sharpness combobox
  ComboBoxSharpness.Enabled := false;
  ComboBoxSharpness.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.Sharpness) do begin
    ComboBoxSharpness.Enabled := true;
    ComboBoxSharpness.Items.Add(gSharpnessArrayText[mCameraRemoteParam.Sharpness[i]]);
  end;

  //Colorspace combobox
  ComboBoxColorspace.Enabled := false;
  ComboBoxColorspace.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.Colorspace) do begin
    ComboBoxColorspace.Enabled := true;
    ComboBoxColorspace.Items.Add(gColorSpaceArrayText[mCameraRemoteParam.ColorSpace[i]]);
  end;

  //ISO combobox
  ComboBoxISO.Enabled := false;
  ComboBoxISO.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.ISO) do begin
    ComboBoxISO.Enabled := true;
    ComboBoxISO.Items.Add(gISOArrayText[mCameraRemoteParam.ISO[i]]);
  end;

  //ExposureCompensation combobox
  ComboBoxExpComp.Enabled := false;
  ComboBoxExpComp.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.ExposureComp) do begin
    ComboBoxExpComp.Enabled := true;
    ComboBoxExpComp.Items.Add(gExposureCompArrayText[mCameraRemoteParam.ExposureComp[i]]);
  end;

  //AV combobox
  ComboBoxAv.Enabled := false;
  ComboBoxAv.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.AV) do begin
    ComboBoxAv.Enabled := true;
    ComboBoxAv.Items.Add(gAvArrayText[mCameraRemoteParam.AV[i]]);
  end;

  //TV combobox
  ComboBoxTv.Enabled := false;
  ComboBoxTv.Items.Clear;
  for i := 0 to high(mCameraRemoteParam.TV) do begin
    ComboBoxTv.Enabled := true;
    ComboBoxTv.Items.Add(gTvArrayText[mCameraRemoteParam.TV[i]]);
  end;

  //Beep checkbox
  CheckBoxBeep.Enabled := false;
  if (high(mCameraRemoteParam.Beep) > 0) then begin
    CheckBoxBeep.Enabled := true;
  end;

  //PictureStyle
  ButtonPictureStyle.Enabled := mCameraCapability.RemoteParamSupported.PictureStyle;
  //WhitebalanceShift
  ButtonWhiteBalanceShift.Enabled := mCameraCapability.RemoteParamSupported.WhiteBalanceShift;

  ComboBoxQualityChange(nil);
end;

//Sets new remote parameters in the VCLs in the parameters tab
procedure TFormRemote.SetRemoteParameters(inParam : RemoteReleaseParametersType);
begin
  ComboBoxQuality.ItemIndex         := ComboBoxQuality.Items.IndexOf(gCompQualArrayText[inParam.CompQuality]);
  ComboBoxQualityPic2.ItemIndex     := ComboBoxQualityPic2.Items.IndexOf(gCompQualArrayText[inParam.CompQualityPic2RAW]);
  ComboBoxSize.ItemIndex            := ComboBoxSize.Items.IndexOf(gImageSizeArrayText[inParam.ImageSize]);
  ComboBoxSizePic2.ItemIndex        := ComboBoxSizePic2.Items.IndexOf(gImageSizeArrayText[inParam.ImageSizePic2RAW]);
  ComboBoxFlash.ItemIndex           := ComboBoxFlash.Items.IndexOf(gStrobeSettingArrayText[inParam.StrobeSetting]);
  ComboBoxFlashComp.ItemIndex       := ComboBoxFlashComp.Items.IndexOf(gStrobeCompSettingArrayText[inParam.StrobeCompSetting]);
  ComboBoxImageMode.ItemIndex       := ComboBoxImageMode.Items.IndexOf(gImageModeArrayText[inParam.ImageMode]);
  ComboBoxDriveMode.ItemIndex       := ComboBoxDriveMode.Items.IndexOf(gDriveModeArrayText[inParam.DriveMode]);
  ComboBoxMeteringMode.ItemIndex    := ComboBoxMeteringMode.Items.IndexOf(gMLWeiModeArrayText[inParam.MLWeiMode]);
  ComboBoxAFMode.ItemIndex          := ComboBoxAFMode.Items.IndexOf(gAFModeArrayText[inParam.AFMode]);
  ComboBoxAFDistance.ItemIndex      := ComboBoxAFDistance.Items.IndexOf(gAFDistanceArrayText[inParam.AFDistance]);
  ComboBoxAFFocusingPoint.ItemIndex := ComboBoxAFFocusingPoint.Items.IndexOf(gAFFocusingPointArrayText[inParam.AFFocusingPoint]);
  CheckBoxAFLight.Checked           := false;
  ComboBoxWhiteBalance.ItemIndex    := ComboBoxWhiteBalance.Items.IndexOf(gWhiteBalanceSetArrayText[inParam.WhiteBalanceSetting]);
  EditWhiteBalanceKelvin.Text       := IntToStr(inParam.WhiteBalanceKelvin);
  ComboBoxPhotoEffect.ItemIndex     := ComboBoxPhotoEffect.Items.IndexOf(gPhotoEffectArrayText[inParam.PhotoEffect]);
  ComboBoxContrast.ItemIndex        := ComboBoxContrast.Items.IndexOf(gContrastArrayText[inParam.Contrast]);
  ComboBoxColorGain.ItemIndex       := ComboBoxColorGain.Items.IndexOf(gColorGainArrayText[inParam.ColorGain]);
  ComboBoxSharpness.ItemIndex       := ComboBoxSharpness.Items.IndexOf(gSharpnessArrayText[inParam.Sharpness]);
  ComboBoxColorspace.ItemIndex      := ComboBoxColorspace.Items.IndexOf(gColorSpaceArrayText[inParam.ColorSpace]);
  ComboBoxISO.ItemIndex             := ComboBoxISO.Items.IndexOf(gISOArrayText[inParam.ISO]);
  ComboBoxAv.ItemIndex              := ComboBoxAv.Items.IndexOf(gAVArrayText[inParam.AV]);
  ComboBoxTv.ItemIndex              := ComboBoxTv.Items.IndexOf(gTVArrayText[inParam.TV]);
  ComboBoxExpComp.ItemIndex         := ComboBoxExpComp.Items.IndexOf(gExposureCompArrayText[inParam.ExposureCompensation]);
  CheckBoxBeep.Checked              := false;
  if (inParam.Beep = RemoteFormatBeepOn) then
  begin
    CheckBoxBeep.Checked := true;
  end;
  if (inParam.AFAssistLight = RemoteFormatAFLightOn) then
  begin
    CheckBoxAFLight.Checked := true;
  end;
  if (inParam.WhiteBalanceShift.WBShiftEnabled) then
  begin
    FormWhitebalanceShift.TrackBarWBShiftAB.Position := inParam.WhiteBalanceShift.AmberBlue;
    FormWhitebalanceShift.TrackBarWBShiftGM.Position := inParam.WhiteBalanceShift.GreenMagenta;
  end;
  if (inParam.PictureStyle.PictureStyleEnabled) then
  begin
    FormPictureStyle.SpinEditContrast.Value  := inParam.PictureStyle.Contrast;
    FormPictureStyle.SpinEditSharpness.Value := inParam.PictureStyle.Sharpness;
    FormPictureStyle.CheckBoxMonochrome.Checked :=
       ((inParam.PictureStyle.MonochromeFilterUsed) and
        (inParam.PictureStyle.MonochromeToneUsed));
    if (inParam.PictureStyle.SaturationUsed) then
    begin
      FormPictureStyle.SpinEditSaturation.Value := inParam.PictureStyle.Saturation;
    end;
    if (inParam.PictureStyle.ColorToneUsed) then
    begin
      FormPictureStyle.SpinEditColorTone.Value := inParam.PictureStyle.ColorTone;
    end;
    if (inParam.PictureStyle.MonochromeFilterUsed) then
    begin
      FormPictureStyle.ComboBoxMonochromeFilter.ItemIndex := Ord(inParam.PictureStyle.MonochromeFilter);
    end;
    if (inParam.PictureStyle.MonochromeToneUsed) then
    begin
      FormPictureStyle.ComboBoxMonochromeTone.ItemIndex := Ord(inParam.PictureStyle.MonochromeTone);
    end;
  end;
end;

//Event when changing the zoom trackbar
procedure TFormRemote.TrackBarZoomChange(Sender: TObject);
//var viewfinder_on : boolean;
begin
  LabelZoomPos.Caption := IntToStr(TrackBarZoom.Position);
  //viewfinder_on := mViewFinderEnabled;
  if ((mReleaseProcessActive) and (mCameraCapability.DoSupportZoom)) then
  begin
    //Turn of Viewfinder temporarily when changing zoom. May not be necessary.
    //if (viewfinder_on) then
    //begin
      //CamRemote.RemoteStopViewfinder;
      //mViewFinderEnabled := false;
      //sleep(1000);
    //end;
    CamRemoteActiveX.RemoteSetZoomPos(TrackBarZoom.Position);
    //if (viewfinder_on) then
    //begin
      //sleep(1000);
      //CamRemote.RemoteStartViewfinder;
      //mViewFinderEnabled := true;
    //end;
  end;
end;

procedure TFormRemote.ButtonViewFinderClick(Sender: TObject);
begin
  FormViewFinder.Show;
end;

procedure TFormRemote.ButtonSetCameraSettingsClick(Sender: TObject);
var remote_parameters : RemoteReleaseParametersType;
    viewfinder_on     : boolean;
begin
  viewfinder_on := mViewFinderEnabled;
  //Turn off viewfinder temporarily when setting the remote parameters,
  //for more information see the manual.
  //Turn viewfinder off
  if (viewfinder_on) then
  begin
    CamRemoteActiveX.RemoteStopViewfinder;
    mViewFinderEnabled := false;
  end;
  //Sets release parameters
  remote_parameters := GetRemoteParameters;
  Log('Sets remote parameters');
  LogRemoteParameters(remote_parameters);
  CamRemoteActiveX.RemoteSetRemoteParams_ImageQuality(Ord(remote_parameters.ImageSize),
                                                      Ord(remote_parameters.CompQuality),
                                                      Ord(remote_parameters.ImageSizePic2RAW),
                                                      Ord(remote_parameters.CompQualityPic2RAW));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamStrobeSetting,
                                         Ord(remote_parameters.StrobeSetting));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamStrobeCompSetting,
                                         Ord(remote_parameters.StrobeCompSetting));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamDriveMode,
                                         Ord(remote_parameters.DriveMode));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamImageMode,
                                         Ord(remote_parameters.ImageMode));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamMLWeiMode,
                                         Ord(remote_parameters.MLWeiMode));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamAFMode,
                                         Ord(remote_parameters.AFMode));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamAFDistance,
                                         Ord(remote_parameters.AFDistance));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamAFFocusingPoint,
                                         Ord(remote_parameters.AFFocusingPoint));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamAFAssistLight,
                                         Ord(remote_parameters.AFAssistLight));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamWhiteBalanceSetting,
                                         Ord(remote_parameters.WhiteBalanceSetting));
  if (remote_parameters.WhiteBalanceSetting = RemoteFormatWBKelvin) then
  begin
    CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamWhiteBalanceKelvin,
                                           remote_parameters.WhiteBalanceKelvin);
  end;
  CamRemoteActiveX.RemoteSetRemoteParams_WhiteBalanceShift(remote_parameters.WhiteBalanceShift.WBShiftEnabled,
                                                           remote_parameters.WhiteBalanceShift.AmberBlue,
                                                           remote_parameters.WhiteBalanceShift.GreenMagenta);
  CamRemoteActiveX.RemoteSetRemoteParams_PictureStyle(remote_parameters.PictureStyle.PictureStyleEnabled,
                                                      remote_parameters.PictureStyle.Contrast,
                                                      remote_parameters.PictureStyle.Sharpness,
                                                      remote_parameters.PictureStyle.Saturation,
                                                      remote_parameters.PictureStyle.SaturationUsed,
                                                      remote_parameters.PictureStyle.ColorTone,
                                                      remote_parameters.PictureStyle.ColorToneUsed,
                                                      Ord(remote_parameters.PictureStyle.MonochromeFilter),
                                                      remote_parameters.PictureStyle.MonochromeFilterUsed,
                                                      Ord(remote_parameters.PictureStyle.MonochromeTone),
                                                      remote_parameters.PictureStyle.MonochromeToneUsed);
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamContrast,
                                         Ord(remote_parameters.Contrast));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamColorGain,
                                         Ord(remote_parameters.ColorGain));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamSharpness,
                                         Ord(remote_parameters.Sharpness));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamColorSpace,
                                         Ord(remote_parameters.ColorSpace));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamISO,
                                         Ord(remote_parameters.ISO));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamAv,
                                         Ord(remote_parameters.Av));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamTv,
                                         Ord(remote_parameters.Tv));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamExposureCompensation,
                                         Ord(remote_parameters.ExposureCompensation));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamPhotoEffect,
                                         Ord(remote_parameters.PhotoEffect));
  CamRemoteActiveX.RemoteSetRemoteParams(RemoteParamBeep,
                                         Ord(remote_parameters.Beep));
  //Turn viewfinder on
  if (viewfinder_on) then
  begin
    CamRemoteActiveX.RemoteStartViewfinder;
    mViewFinderEnabled := true;
  end;
end;

//Gets remote parameters from the VCL in the remote tab
function TFormRemote.GetRemoteParameters : RemoteReleaseParametersType;
begin
  //Clear return structure
  FillChar(result, sizeof(RemoteReleaseParametersType), #0);
  if (ComboBoxQuality.Enabled) then
    result.CompQuality  := mCameraRemoteParam.CompQuality[ComboBoxQuality.ItemIndex]
  else
    result.CompQuality := RemoteFormatQualityNotUsed;

  if (ComboBoxQualityPic2.Enabled) then
    result.CompQualityPic2RAW  := mCameraRemoteParam.CompQualityPic2RAW[ComboBoxQualityPic2.ItemIndex]
  else
    result.CompQualityPic2RAW := RemoteFormatQualityNotUsed;

  if (ComboBoxSize.Enabled) then
    result.ImageSize := mCameraRemoteParam.ImageSize[ComboBoxSize.ItemIndex]
  else
    result.ImageSize := RemoteFormatSizeNotUsed;

  if (ComboBoxSizePic2.Enabled) then
    result.ImageSizePic2RAW := mCameraRemoteParam.ImageSizePic2RAW[ComboBoxSizePic2.ItemIndex]
  else
    result.ImageSizePic2RAW := RemoteFormatSizeNotUsed;

  if (ComboBoxFlash.Enabled) then
    result.StrobeSetting := mCameraRemoteParam.StrobeSetting[ComboBoxFlash.ItemIndex]
  else
    result.StrobeSetting := RemoteFormatFlashNotUsed;

  if (ComboBoxFlashComp.Enabled) then
    result.StrobeCompSetting := mCameraRemoteParam.StrobeCompSetting[ComboBoxFlashComp.ItemIndex]
  else
    result.StrobeCompSetting := RemoteFormatFlashCompNotUsed;

  if (ComboBoxImageMode.Enabled) then
    result.ImageMode := mCameraRemoteParam.ImageMode[ComboBoxImageMode.ItemIndex]
  else
    result.ImageMode := RemoteFormatShootingModeNotUsed;

  if (ComboBoxDriveMode.Enabled) then
    result.DriveMode := mCameraRemoteParam.DriveMode[ComboBoxDriveMode.ItemIndex]
  else
    result.DriveMode := RemoteFormatDriveModeNotUsed;

  if (ComboBoxMeteringMode.Enabled) then
    result.MLWeiMode := mCameraRemoteParam.MLWeiMode[ComboBoxMeteringMode.ItemIndex]
  else
    result.MLWeiMode := RemoteFormatMLWeiNotUsed;

  if (ComboBoxAFMode.Enabled) then
    result.AFMode := mCameraRemoteParam.AFMode[ComboBoxAFMode.ItemIndex]
  else
    result.AFMode := RemoteFormatAFModeNotUsed;

  if (ComboBoxAFDistance.Enabled) then
    result.AFDistance := mCameraRemoteParam.AFDistance[ComboBoxAFDistance.ItemIndex]
  else
    result.AFDistance := RemoteFormatAFDistNotUsed;

  if (ComboBoxAFFocusingPoint.Enabled) then
    result.AFFocusingPoint := mCameraRemoteParam.AFFocusingPoint[ComboBoxAFFocusingPoint.ItemIndex]
  else
    result.AFFocusingPoint := RemoteFormatAFFocusingPointNotUsed;

  if (CheckBoxAFLight.Enabled) then
  begin
    if (CheckBoxAFLight.Checked) then
    begin
      result.AFAssistLight := RemoteFormatAFLightOn;
    end else begin
      result.AFAssistLight := RemoteFormatAFLightOff;
    end;
  end else begin
    result.AFAssistLight := RemoteFormatAFLightNotUsed;
  end;

  if (ComboBoxWhiteBalance.Enabled) then
    result.WhiteBalanceSetting  := mCameraRemoteParam.WhiteBalanceSetting[ComboBoxWhiteBalance.ItemIndex]
  else
    result.WhiteBalanceSetting  := RemoteFormatWBNotUsed;

  result.WhiteBalanceKelvin := 0;
  if (ComboBoxWhiteBalance.Text = KELVIN_TEXT) then
  begin
    result.WhiteBalanceKelvin := StrToInt(EditWhiteBalanceKelvin.Text);
  end;

  if (ComboBoxPhotoEffect.Enabled) then
    result.PhotoEffect := mCameraRemoteParam.PhotoEffect[ComboBoxPhotoEffect.ItemIndex]
  else
    result.PhotoEffect := RemoteFormatPhotoEffectNotUsed;

  if (ComboBoxContrast.Enabled) then
    result.Contrast := mCameraRemoteParam.Contrast[ComboBoxContrast.ItemIndex]
  else
    result.Contrast := RemoteFormatLevelNotUsed;

  if (ComboBoxColorGain.Enabled) then
    result.ColorGain := mCameraRemoteParam.ColorGain[ComboBoxColorGain.ItemIndex]
  else
    result.ColorGain := RemoteFormatLevelNotUsed;

  if (ComboBoxSharpness.Enabled) then
    result.Sharpness := mCameraRemoteParam.Sharpness[ComboBoxSharpness.ItemIndex]
  else
    result.Sharpness := RemoteFormatLevelNotUsed;

  if (ComboBoxColorspace.Enabled) then
    result.ColorSpace := mCameraRemoteParam.ColorSpace[ComboBoxColorspace.ItemIndex]
  else
    result.ColorSpace := RemoteFormatColorSpaceNotUsed;

  if (ComboBoxISO.Enabled) then
    result.ISO := mCameraRemoteParam.ISO[ComboBoxISO.ItemIndex]
  else
    result.ISO := RemoteFormatISONotUsed;

  if (ComboBoxAV.Enabled) then
    result.AV := mCameraRemoteParam.AV[ComboBoxAv.ItemIndex]
  else
    result.AV := RemoteFormatAVNotUsed;

  if (ComboBoxTV.Enabled) then
    result.TV := mCameraRemoteParam.TV[ComboBoxTv.ItemIndex]
  else
    result.TV := RemoteFormatTVNotUsed;

  if (ComboBoxExpComp.Enabled) then
    result.ExposureCompensation := mCameraRemoteParam.ExposureComp[ComboBoxExpComp.ItemIndex]
  else
    result.ExposureCompensation := RemoteFormatExposureCompNotUsed;

  if (CheckBoxBeep.Enabled) then
  begin
    if (CheckBoxBeep.Checked) then
    begin
      result.Beep := RemoteFormatBeepOn;
    end else begin
      result.Beep := RemoteFormatBeepOff;
    end;
  end else begin
    result.Beep := RemoteFormatBeepNotUsed;
  end;
  //Update whitebalance shift
  result.WhiteBalanceShift.WBShiftEnabled := ButtonWhiteBalanceShift.Enabled;
  result.WhiteBalanceShift.AmberBlue      := FormWhitebalanceShift.TrackBarWBShiftAB.Position;
  result.WhiteBalanceShift.GreenMagenta   := FormWhitebalanceShift.TrackBarWBShiftGM.Position;
  //Update picture style
  result.PictureStyle.PictureStyleEnabled := ButtonPictureStyle.Enabled;
  result.PictureStyle.Contrast := FormPictureStyle.SpinEditContrast.Value;
  result.PictureStyle.Sharpness := FormPictureStyle.SpinEditSharpness.Value;
  result.PictureStyle.SaturationUsed :=
    not FormPictureStyle.CheckBoxMonochrome.Checked;
  result.PictureStyle.Saturation := FormPictureStyle.SpinEditSaturation.Value;
  result.PictureStyle.ColorToneUsed :=
    not FormPictureStyle.CheckBoxMonochrome.Checked;
  result.PictureStyle.ColorTone := FormPictureStyle.SpinEditColorTone.Value;
  result.PictureStyle.MonochromeFilterUsed :=
    FormPictureStyle.CheckBoxMonochrome.Checked;
  if (FormPictureStyle.ComboBoxMonochromeFilter.ItemIndex <> -1) then
  begin
    result.PictureStyle.MonochromeFilter :=
      MonochromeFilterType(FormPictureStyle.ComboBoxMonochromeFilter.ItemIndex);
  end;
  result.PictureStyle.MonochromeToneUsed :=
    FormPictureStyle.CheckBoxMonochrome.Checked;
  if (FormPictureStyle.ComboBoxMonochromeTone.ItemIndex <> -1) then
  begin
    result.PictureStyle.MonochromeTone :=
      MonochromeToneType(FormPictureStyle.ComboBoxMonochromeTone.ItemIndex);
  end;
end;

procedure TFormRemote.LogRemoteParameters(RemoteParameters : RemoteReleaseParametersType);
begin
  //Log remote properties
  Log('Logs remote parameters');
  Log('CompQuality          = ' + inttostr(ord(RemoteParameters.CompQuality)));
  Log('CompQualityPic2RAW   = ' + inttostr(ord(RemoteParameters.CompQualityPic2RAW)));
  Log('ImageSize            = ' + inttostr(ord(RemoteParameters.ImageSize)));
  Log('ImageSizePic2RAW     = ' + inttostr(ord(RemoteParameters.ImageSizePic2RAW)));
  Log('StrobeSetting        = ' + inttostr(ord(RemoteParameters.StrobeSetting)));
  Log('StrobeCompSetting    = ' + inttostr(ord(RemoteParameters.StrobeCompSetting)));
  Log('DriveMode            = ' + inttostr(ord(RemoteParameters.DriveMode)));
  Log('ImageMode            = ' + inttostr(ord(RemoteParameters.ImageMode)));
  Log('MLWeiMode            = ' + inttostr(ord(RemoteParameters.MLWeiMode)));
  Log('AFMode               = ' + inttostr(ord(RemoteParameters.AFMode)));
  Log('AFDistance           = ' + inttostr(ord(RemoteParameters.AFDistance)));
  Log('AFFocusingPOint      = ' + inttostr(ord(RemoteParameters.AFFocusingPoint)));
  Log('AFAssistLight        = ' + inttostr(ord(RemoteParameters.AFAssistLight)));
  Log('WhiteBalanceSetting  = ' + inttostr(ord(RemoteParameters.WhiteBalanceSetting)));
  Log('WhiteBalanceKelvin   = ' + inttostr(ord(RemoteParameters.WhiteBalanceKelvin)));
  Log('WBShiftEnabled       = ' + inttostr(ord(RemoteParameters.WhiteBalanceShift.WBShiftEnabled)));
  Log('WBShiftAmberBlue     = ' + inttostr(ord(RemoteParameters.WhiteBalanceShift.AmberBlue)));
  Log('WBShiftGreenMagenta  = ' + inttostr(ord(RemoteParameters.WhiteBalanceShift.GreenMagenta)));
  Log('PictureStyleEnabled  = ' + inttostr(ord(RemoteParameters.PictureStyle.PictureStyleEnabled)));
  Log('Contrast             = ' + inttostr(ord(RemoteParameters.PictureStyle.Contrast)));
  Log('Sharpness            = ' + inttostr(ord(RemoteParameters.PictureStyle.Sharpness)));
  Log('Saturation           = ' + inttostr(ord(RemoteParameters.PictureStyle.Saturation)));
  Log('SaturationUsed       = ' + inttostr(ord(RemoteParameters.PictureStyle.SaturationUsed)));
  Log('ColorTone            = ' + inttostr(ord(RemoteParameters.PictureStyle.ColorTone)));
  Log('ColorToneUsed        = ' + inttostr(ord(RemoteParameters.PictureStyle.ColorToneUsed)));
  Log('MonochromeFilter     = ' + inttostr(ord(RemoteParameters.PictureStyle.MonochromeFilter)));
  Log('MonochromeFilterUsed = ' + inttostr(ord(RemoteParameters.PictureStyle.MonochromeFilterUsed)));
  Log('MonochromeTone       = ' + inttostr(ord(RemoteParameters.PictureStyle.MonochromeTone)));
  Log('MonochromeToneUsed   = ' + inttostr(ord(RemoteParameters.PictureStyle.MonochromeToneUsed)));
  Log('PhotoEffect          = ' + inttostr(ord(RemoteParameters.PhotoEffect)));
  Log('Contrast             = ' + inttostr(ord(RemoteParameters.Contrast)));
  Log('ColorGain            = ' + inttostr(ord(RemoteParameters.ColorGain)));
  Log('Sharpness            = ' + inttostr(ord(RemoteParameters.Sharpness)));
  Log('ColorSpace           = ' + inttostr(ord(RemoteParameters.ColorSpace)));
  Log('ISO                  = ' + inttostr(ord(RemoteParameters.ISO)));
  Log('Av                   = ' + inttostr(ord(RemoteParameters.Av)));
  Log('Tv                   = ' + inttostr(ord(RemoteParameters.Tv)));
  Log('ExposureCompensation = ' + inttostr(ord(RemoteParameters.ExposureCompensation)));
  Log('Beep                 = ' + inttostr(ord(RemoteParameters.Beep)));
end;

//Gets application version text
function TFormRemote.Versiontext : string;
var
  size  : DWord;
  size2 : DWord;
  pt    : Pointer;
  pt2   : Pointer;
begin
  //Get the version of application
  result := '';
  size   := GetFileVersionInfoSize(PChar(ParamStr(0)), size2);
  if (size > 0) then
  begin
    GetMem(pt, size);
    try
       GetFileVersionInfo(PChar(ParamStr(0)), 0, size, pt);
       // show the fixed information
       VerQueryValue(pt, '\', pt2, size2);
       with TVSFixedFileInfo(pt2^) do
       begin
         result := result + IntToStr(HiWord(dwFileVersionMS)) + '.';
         result := result + IntToStr(LoWord(dwFileVersionMS)) + '.';
         result := result + IntToStr(HiWord(dwFileVersionLS));
         result := result + ' Build:' + IntToStr(LoWord(dwFileVersionLS));
       end;
    finally
      FreeMem(pt);
    end;
  end;
end;

//Separate application exception handler, making it possible to log
//all exceptions.
procedure TFormRemote.AppException(Sender: TObject; E: Exception);
begin
  log('AppException. Exception=' + E.Message);
  Application.ShowException(E);
end;

//Function to get the filename of the log-file
function TFormRemote.GetLogFileName : string;
var file_dir : string;
begin
  file_dir := ParamStr(0);
  file_dir := extractFileDir(file_dir);
  result   := file_dir + LOG_FILE_NAME;
end;

//Write OS information to log
procedure TFormRemote.WriteVerToLog;
var
  ver_info : TOSVERSIONINFO;
  str      : String;
  i        : Word;
begin
  ver_info.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
  if GetVersionEx(ver_info) then begin
    Log('Size of OS-version text:' + IntToStr(ver_info.dwOSVersionInfoSize));
    Log('Major OS-version:'        + IntToStr(ver_info.dwMajorVersion));
    Log('Minor OS-version:'        + IntToStr(ver_info.dwMinorVersion));
    Log('Build of OS-version:'     + IntToStr(ver_info.dwBuildNumber));
    case ver_info.dwPlatformId of
      VER_PLATFORM_WIN32s         : Log('Win16 running Win32s');
      VER_PLATFORM_WIN32_WINDOWS  : Log('Win32 Windows, probably Win95');
      VER_PLATFORM_WIN32_NT       : Log('WinNT, full 32-bit');
    end;
    str := '';
    for i := 0 to 127 do
    begin
      str := str + ver_info.szCSDVersion[i];
    end;
    Log('Additional OS data:' + str);
  end;
end;

//Procedure for writing a string to the log
procedure TFormRemote.Log(Text : string);
var buffer : array[1..512] of char;
    i      : integer;
begin
  if (CheckBoxDebug.Checked) then
  begin
    Text := DateTimeToStr(now) + ':' + Text;
    for i := 1 to length(Text) do
    begin
      buffer[i] := Text[i];
    end;
    buffer[length(Text) + 1] := #13;
    buffer[length(Text) + 2] := #10;
    mLogFile.Write(buffer, length(Text) + 2);
  end;
end;

//Procedure to connect to a camera
procedure TFormRemote.ConnectToCamera;
var i : integer;
begin
  //See if already conected
  if (not mConnToCamera) then
  begin
    try
      CamRemoteActiveX.OpenCameraEnumeration;
      if ((CamRemoteActiveX.OpenCameraEnumeration_GetPowerShotNr = 0) and
          (CamRemoteActiveX.OpenCameraEnumeration_GetEOSNr = 0)) then
      begin
        raise ECamException.Create('No camera is connected to the computer');
      end;
      FormSelectCamera.ModalResult := mrOK;
      FormSelectCamera.ComboBoxCameras.Items.Clear;
      //Are there any PowerShot cameras connected?
      if (CamRemoteActiveX.OpenCameraEnumeration_GetPowerShotNr > 0) then
      begin
        if (CamRemoteActiveX.OpenCameraEnumeration_GetPowerShotNr > 1) then
        begin
          //Select which camera to connect to
          for i := 0 to (CamRemoteActiveX.OpenCameraEnumeration_GetPowerShotNr - 1) do
          begin
            FormSelectCamera.ComboBoxCameras.Items.Add(CamRemoteActiveX.OpenCameraEnumeration_GetModel(Ord(CamModPowerShot), i));
          end;
          FormSelectCamera.ComboBoxCameras.ItemIndex := 0;
          if (FormSelectCamera.ShowModal = mrOK) then
          begin
            CamRemoteActiveX.Connect(Ord(CamModPowerShot),
                                     FormSelectCamera.ComboBoxCameras.ItemIndex);
          end;
        end else begin
          //Only one PowerShot camera connected. Select that camera
          CamRemoteActiveX.Connect(Ord(CamModPowerShot), 0);
        end;
      end else begin
        //No PowerShot cameras. Connect to EOS camera
        if (CamRemoteActiveX.OpenCameraEnumeration_GetEOSNr > 1) then
        begin
          //Select which camera to connect to
          for i := 0 to (CamRemoteActiveX.OpenCameraEnumeration_GetEOSNr - 1) do
          begin
            FormSelectCamera.ComboBoxCameras.Items.Add(CamRemoteActiveX.OpenCameraEnumeration_GetModel(Ord(CamModEOS), i));
          end;
          FormSelectCamera.ComboBoxCameras.ItemIndex := 0;
          if (FormSelectCamera.ShowModal = mrOK) then
          begin
            CamRemoteActiveX.Connect(Ord(CamModEOS),
                                     FormSelectCamera.ComboBoxCameras.ItemIndex);
          end;
        end else begin
          //Only one EOS camera connected. Select that camera
            CamRemoteActiveX.Connect(Ord(CamModEOS), 0);
        end;
      end;
      if (FormSelectCamera.ModalResult <> mrOK) then
      begin
        Exit;
      end;
      mCameraInfo.CameraModel :=
        CamModType(CamRemoteActiveX.Connect_CameraModel);
      mCameraInfo.CameraModelName :=
        CamRemoteActiveX.Connect_CameraModelName;
      mCameraInfo.OwnerName :=
        CamRemoteActiveX.Connect_OwnerName;
      mCameraInfo.Battery.BatterySource :=
        BatterySourceType(CamRemoteActiveX.Connect_Battery_BatterySource);
      mCameraInfo.Battery.BatteryStatus :=
        BatteryStatusType(CamRemoteActiveX.Connect_Battery_BatteryStatus);
      //Update battery status
      LabelBatterySource.Caption := '-';
      case mCameraInfo.Battery.BatterySource of
        BatterySourceAC     : LabelBatterySource.Caption := 'AC';
        BatterySourceLitium : LabelBatterySource.Caption := 'Litium';
        BatterySourceNiMH   : LabelBatterySource.Caption := 'NiMH';
        BatterySourceNiCD   : LabelBatterySource.Caption := 'NiCD';
        BatterySourceAlMN   : LabelBatterySource.Caption := 'AlMN';
      end;
      ProgressBarBattery.Position := 1;
      case mCameraInfo.Battery.BatteryStatus of
        BatteryStatusNormal    : ProgressBarBattery.Position := 5;
        BatteryStatusWeak      : ProgressBarBattery.Position := 3;
        BatteryStatusSafetyLow : ProgressBarBattery.Position := 2;
      end;
      //Update BodyID
      LabelBodyIDValue.Caption := Format('%u', [CamRemoteActiveX.GetBodyID]);
      //Update owner name
      EditOwnerName.Text := mCameraInfo.OwnerName;

      StatusBar.SimpleText := 'Camera is connected';
      mConnToCamera := true;
      Log('Camera model='        + mCameraInfo.CameraModelName);
      Log('Camera name='         + mCameraInfo.OwnerName);
      Log('Camera connected');
    except
      on E:exception do
        begin
          log('Connect_to_powershot. Exception=' + E.Message);
          StatusBar.SimpleText := 'Connection to camera failed';
          mConnToCamera        := false;
          raise;
        end;
    end;
  end;
end;

//Procedure to disconnect from a camera
procedure TFormRemote.DisconnectFromCamera;
begin
  //See if not connected
  if (mConnToCamera) then
  begin
    try
      mConnToCamera := false;
      CamRemoteActiveX.Disconnect;
      CamRemoteActiveX.CloseCameraEnumeration;
      mCamerasEnumerated := false;
      StatusBar.SimpleText := 'Camera is disconnected';
      Log('Camera is disconnected');
    except
      on E:exception do
        begin
          log('Disconnect_from_powershot. Exception=' + E.Message);
          StatusBar.SimpleText := 'Camera communication error';
          raise;
        end;
    end;
  end else begin
    StatusBar.SimpleText := 'Camera already disconnected';
  end;
end;

procedure TFormRemote.CheckBoxDebugClick(Sender: TObject);
var default_string : string;
begin
  if FormRemote.Visible then
  begin
    if (not CheckBoxDebug.Checked) then
    begin
      default_string := Format('Debug is disabled. The logfile (%s) will be removed',
                               [GetLogFileName]);
      MessageDlg(default_string, mtInformation, [mbYes], 0);
      mLogFile.Free;
      DeleteFile(GetLogFileName);
    end else begin
      default_string := Format('Debug is enabled. The logfile (%s) will be created',
                               [GetLogFileName]);
      MessageDlg(default_string, mtInformation, [mbYes], 0);
      mLogFile := TFileStream.Create(GetLogFileName, (fmCreate or fmShareDenyWrite));
    end;
  end;
end;

//Remote process event handler
procedure TFormRemote.TimerTakePictureTimer(Sender: TObject);
begin
   TimerTakePicture.Enabled := false;
   ButtonTakePicClick(Application);
end;

procedure TFormRemote.TimerReceivePicturesTimer(Sender: TObject);
begin
  //Skip check for thumbnail, since it is not of interest within this application.
  //Only check when new pictures are available.
  if (mPicturesToReceive > 0) then
  begin
    mPicturesToReceive := mPicturesToReceive - 1;
    PictureTaken;
  end;
end;

procedure TFormRemote.ButtonSetTimeClick(Sender: TObject);
var year        : word;
    month       : word;
    day         : word;
    hour        : word;
    minutes     : word;
    second      : word;
    millisecond : word;
begin
  DecodeDateTime(Now, year, month, day, hour, minutes, second, millisecond);
  CamRemoteActiveX.SetTimeInCamera(year, month, day, hour, minutes, second, millisecond);
end;

procedure TFormRemote.ButtonSetOwnerNameClick(Sender: TObject);
begin
   CamRemoteActiveX.SetOwnerName(EditOwnerName.Text);
end;

procedure TFormRemote.ButtonFormatCardClick(Sender: TObject);
begin
  CamRemoteActiveX.FormatCameraCard;
end;

procedure TFormRemote.ButtonLockAFClick(Sender: TObject);
begin
  if (not mCameraCapability.DoSupportAfLockUnlock) then
  begin
    raise ECamException.Create('Camera does not support the AF Lock function');
  end;
  if (ButtonLockAF.Caption = LOCK_AF_TXT) then
  begin
    ButtonLockAF.Caption := UNLOCK_AF_TXT;
    CamRemoteActiveX.RemoteAFLock(true);
  end else begin
    ButtonLockAF.Caption := LOCK_AF_TXT;
    CamRemoteActiveX.RemoteAFLock(false);
  end;
end;

procedure TFormRemote.ComboBoxWhiteBalanceChange(Sender: TObject);
begin
  EditWhiteBalanceKelvin.Enabled := (ComboBoxWhiteBalance.Text = KELVIN_TEXT);
end;

procedure TFormRemote.EditWhiteBalanceKelvinChange(Sender: TObject);
begin
  //Check for valid numbers
  StrToInt(EditWhiteBalanceKelvin.Text);
end;

procedure TFormRemote.ComboBoxQualityChange(Sender: TObject);
begin
  if ((ComboBoxSizePic2.Items.Count > 0) and
      (ComboBoxQualityPic2.Items.Count > 0)) then
  begin
    ComboBoxSizePic2.Enabled    := (ComboBoxQuality.Text = RAW_TEXT);
    ComboBoxQualityPic2.Enabled := (ComboBoxQuality.Text = RAW_TEXT);
    if (not ComboBoxSizePic2.Enabled) then
    begin
      ComboBoxSizePic2.Text := '';
    end;
    if (not ComboBoxQualityPic2.Enabled) then
    begin
      ComboBoxQualityPic2.Text := '';
    end;
  end;
end;

procedure TFormRemote.ButtonPictureStyleClick(Sender: TObject);
begin
  FormPictureStyle.ShowModal;
end;

procedure TFormRemote.UpdateAvailableShots;
begin
  if (mReleaseProcessActive) then
  begin
     LabelPicturesLeft.Caption := 'Pictures left: ' + IntToStr(CamRemoteActiveX.RemoteGetNumberOfAvailableShots);
  end;
end;

procedure TFormRemote.ButtonWhiteBalanceShiftClick(Sender: TObject);
begin
  FormWhitebalanceShift.ShowModal;
end;

procedure TFormRemote.ButtonLoadParametersClick(Sender: TObject);
var file_name : string;
begin
{$IFNDEF VER130}
  file_name := GetEnvironmentVariable('Temp') + '\' + REMOTE_SUBKEY + '\' +
               mCameraInfo.CameraModelName + ' ' + REMOTE_PARAM_FILE;
{$ELSE}
  file_name := ExtractFileDir(ParamStr(0)) + '\' +
               mCameraInfo.CameraModelName + ' ' + REMOTE_PARAM_FILE;
{$ENDIF}
  CamRemoteActiveX.RemoteLoadCameraRemoteParams(file_name);
  ButtonGetCamSetClick(Sender);
end;

procedure TFormRemote.ButtonSaveParametersClick(Sender: TObject);
var file_name : string;
begin
  ButtonSetCameraSettingsClick(Sender);
{$IFNDEF VER130}
  file_name := GetEnvironmentVariable('Temp') + '\' + REMOTE_SUBKEY + '\' +
               mCameraInfo.CameraModelName + ' ' + REMOTE_PARAM_FILE;
{$ELSE}
  file_name := ExtractFileDir(ParamStr(0)) + '\' +
               mCameraInfo.CameraModelName + ' ' + REMOTE_PARAM_FILE;
{$ENDIF}
  CamRemoteActiveX.RemoteSaveCameraRemoteParams(file_name);
end;

//Camera event handler
procedure TFormRemote.CamRemoteActiveXEvent(ASender: TObject; Severity,
  Event: TOleEnum);
var camera_text   : string;
    text_message  : string;
    message_out   : MessageQueueElementType;
begin
  FormRemote.Log('Callback Severity=' + inttostr(Ord(Severity)) +
                 'Event =' + inttostr(Ord(Event)));
  //Do not handle information message, not of interest
  if (Severity = Ord(EventSeverityInfo)) then
    exit;
  camera_text := 'Camera event-';
  case Severity of
    Ord(EventSeverityWarning) : text_message := 'Camera warning event';
    Ord(EventSeverityClosing) : text_message := 'Camera shutdown event';
  end;
  case Event of
    Ord(EventBatteryLevelNormal)           : camera_text := camera_text + 'Battery level normal';
    Ord(EventBatteryLevelWeak)             : camera_text := camera_text + 'Battery level weak';
    Ord(EventBatteryLevelSafetyLow)        : camera_text := camera_text + 'Battery level safety low';
    Ord(EventBatteryLevelLB)               : camera_text := camera_text + 'Battery level LB';
    Ord(EventDialChanged)                  : camera_text := camera_text + 'Dial changed';
    Ord(EventCFGateOpened)                 : camera_text := camera_text + 'CF gate opened';
    Ord(EventBatteryCoverOpened)           : camera_text := camera_text + 'Battery cover opened';
    Ord(EventConnectionDisappeared)        : camera_text := camera_text + 'Camera connection disappeared';
    Ord(EventUnrecoverableError)           : camera_text := camera_text + 'Unrecoverable error';
    Ord(EventUnkonwnCommandReceived)       : camera_text := camera_text + 'Unknown command received';
    Ord(EventRemoteParameterChanged)       : camera_text := camera_text + 'Remote parameter changed';
    Ord(EventRemoteCaptureError)           : camera_text := camera_text + 'Capture error';
    Ord(EventRemoteShutdownReasonNotKnown) : camera_text := camera_text + 'Shutdown reason not known';
  end;
  //Send the event text to the message thread
  message_out.MessageText   := text_message;
  message_out.Caption       := camera_text;
  message_out.Style         := MB_OK or MB_ICONWARNING;
  message_out.Critical      := false;
  FormRemote.MessageThread.AddMessage(message_out);
end;

//Camera remote event handler
procedure TFormRemote.CamRemoteActiveXRemoteEvent(ASender: TObject; Event,
  EventValid: TOleEnum; NumOfEvents: Integer; TypeOfPicture: TOleEnum);
begin
  FormRemote.Log('Remote callback event=' + inttostr(Ord(Event)));
  //Check if the release button was pressed on the camera.
  //This applies to PowerShot cameras. EOS cameras do not generate
  //this event when pressing the shutter butten. Instead the EOS camera
  //takes the picture and generates a RemoteEventCallbackReleaseStart event.
  case Event of
    Ord(RemoteEventCallbackCamReleaseOn) :
      begin
        TimerTakePicture.Enabled := true;
      end;
    Ord(RemoteEventCallbackReleaseComplete) :
      begin
        case REMOTE_MODE of
          ReleaseDataKingTakeBothThumbAndPic :
            begin
              mThumbnailsToReceive := mThumbnailsToReceive + NumOfEvents;
              mPicturesToReceive   := mPicturesToReceive   + NumOfEvents;
            end;
          ReleaseDataKindTakeOnlyPicture :
            begin
              mPicturesToReceive   := mPicturesToReceive   + NumOfEvents;
            end;
        end;
      end;
    Ord(RemoteEventCallbackReleaseImageReady) :
      begin
        case TypeOfPicture of
          Ord(TypeOfPictureThumbnail) : mThumbnailsToReceive := mThumbnailsToReceive + 1;
          Ord(TypeOfPicturePicture)   : mPicturesToReceive   := mPicturesToReceive   + 1;
        end;
      end;
    Ord(RemoteEventCallbackChangedByUI) :
      begin
        if (FormRemote.ReleaseProcessActive) then
        begin
          FormRemote.ButtonGetCamSetClick(Application);
        end;
      end;
  end;
end;

//Get picture after taking a picture event handler
procedure TFormRemote.CamRemoteActiveXRemoteGetPictureEvent(
  ASender: TObject; PercentageDone: Integer);
begin
  FormRemote.StatusBar.SimpleText := 'Receiving the picture to disc';
  FormRemote.ProgressBar.Position := PercentageDone;
end;

//Progress event handler during probing
procedure TFormRemote.CamRemoteActiveXRemoteProbeParamEvent(
  ASender: TObject; PercentageDone: Integer);
begin
  FormRemote.StatusBar.SimpleText := 'Probes remote parameters. ' +
                                     IntToStr(PercentageDone) +
                                     ' percent done.';
  FormRemote.ProgressBar.Position := PercentageDone;
end;

//Receiving picture event handler when taking a picture
procedure TFormRemote.CamRemoteActiveXRemoteTakePictureEvent(
  ASender: TObject; PercentageDone: Integer);
begin
  FormRemote.StatusBar.SimpleText := 'Handles the picture';
  FormRemote.ProgressBar.Position := PercentageDone;
end;

//Viewfinder event handler.
procedure TFormRemote.CamRemoteActiveXViewfinderEvent(ASender: TObject;
  const ViewfinderFileName: WideString);
var jpg : TJpegImage;
begin
  if (not mNewViewfinder) then
  begin
    jpg := TJpegImage.Create;
     //Get the 320x240 picture and save it into a Bitmap
    try
      jpg.LoadFromFile(ViewfinderFileName);
      mViewfinderPic.Assign(jpg);
    finally
      jpg.Free;
    end;
    mNewViewfinder := true;
  end;
  DeleteFile(ViewfinderFileName);
end;

end.


